@@ -1,31 +1,8 @@
 # For complete documentation of this file, please see Geany's main documentation
-[styling]
-# foreground;background;bold;italic
-default=default
-comment=comment
-commentline=comment
-commentdoc=commentdoc
-number=number
-word=word
-word2=word2
-string=string
-character=string
-uuid=0x404080
-preprocessor=preprocessor
-operator=operator
-identifier=default
-stringeol=stringeol
-# @"verbatim"
-verbatim=0x101030
-# (/regex/)
-regex=0x105090
-commentlinedoc=commentdoc,bold
-commentdockeyword=commentdoc,bold,italic
-commentdockeyworderror=commentdoc
-globalclass=type
+[styling=C]
 
 [keywords]
-primary=abstract assert break case catch class const continue default do else extends final finally for future generic goto if implements import inner instanceof interface native new outer package private protected public rest return static super switch synchronized this throw throws transient try var volatile while true false null
+primary=abstract assert break case catch class const continue default do else enum extends final finally for future generic goto if implements import inner instanceof interface native new outer package private protected public rest return static strictfp super switch synchronized this throw throws transient try var volatile while true false null
 secondary=boolean byte char double float int long short void
 # documentation keywords for javadoc
 doccomment=author deprecated exception param return see serial serialData serialField since throws todo version
@@ -34,12 +11,14 @@ typedefs=
 
 [settings]
 # default extension used when saving files
-#extension=java
+extension=java
 
 # the following characters are these which a "word" can contains, see documentation
 #wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
-# if only single comment char is supported like # in this file, leave comment_close blank
+# single comments, like # in this file
+comment_single=//
+# multiline comments
 comment_open=/*
 comment_close=*/
 
@@ -54,6 +33,10 @@ comment_use_indent=true
 # context action command (please see Geany's main documentation for details)
 context_action_cmd=
 
+[indentation]
+#width=4
+# 0 is spaces, 1 is tabs, 2 is tab & spaces
+#type=1
 
 [build_settings]
 # %f will be replaced by the complete filename
