@@ -1,38 +1,54 @@
 # For complete documentation of this file, please see Geany's main documentation
 [styling]
-# foreground;background;bold;italic
-default=0x000000;0xffffff;false;false
-commentline=0xd00000;0xffffff;false;false
-number=0x007f00;0xffffff;false;false
-word=0x111199;0xffffff;true;false
-string=0xff901e;0xffffff;false;false
-character=0x404000;0xffffff;false;false
-operator=0x301010;0xffffff;false;false
-identifier=0x000000;0xffffff;false;false
-backticks=0x000000;0xd0d0d0;false;false
-param=0x009f00;0xffffff;false;false
-scalar=0x105090;0xffffff;false;false
+# Edit these in the colorscheme .conf file instead
+default=default
+commentline=comment_line
+number=number_1
+word=keyword_1
+string=string_1
+character=character
+operator=operator
+identifier=identifier
+backticks=backticks
+param=parameter
+scalar=identifier_1
+error=error
+here_delim=here_doc
+here_q=here_doc
 
 [keywords]
 primary=break case continue do done elif else esac eval exit export fi for function goto if in integer return set shift then until while
 
 
 [settings]
+# default extension used when saving files
+extension=sh
+
 # the following characters are these which a "word" can contains, see documentation
 #wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
-# if only single comment char is supported like # in this file, leave comment_close blank
-comment_open=#
-comment_close=
+# single comments, like # in this file
+comment_single=#
+# multiline comments
+#comment_open=
+#comment_close=
 
 # set to false if a comment character/string should start a column 0 of a line, true uses any
-# indention of the line, e.g. setting to true causes the following on pressing CTRL+d
+# indentation of the line, e.g. setting to true causes the following on pressing CTRL+d
 	#command_example();
 # setting to false would generate this
 #	command_example();
 # This setting works only for single line comments
 comment_use_indent=true
 
+# context action command (please see Geany's main documentation for details)
+context_action_cmd=
+
+[indentation]
+#width=4
+# 0 is spaces, 1 is tabs, 2 is tab & spaces
+#type=1
+
 [build_settings]
 # %f will be replaced by the complete filename
 # %e will be replaced by the filename without extension
