@@ -1,8 +1,8 @@
 /*
  *      geany.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: geany.h 5023 2010-06-12 13:42:50Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /* Main header - should be included first in all source files.
@@ -27,12 +25,14 @@
 #ifndef GEANY_H
 #define GEANY_H
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <gtk/gtk.h>
 
+G_BEGIN_DECLS
+
+#if defined(HAVE_CONFIG_H) && defined(GEANY_PRIVATE)
+#	include "config.h"
+#endif
+
 #include "tm_tagmanager.h"
 
 #ifndef PLAT_GTK
@@ -47,8 +47,11 @@
  * listed in the documentation should not be changed */
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
 #define GEANY_TEMPLATES_SUBDIR			"templates"
-#define GEANY_CODENAME					"Disra"
+#define GEANY_CODENAME					"Sakai"
 #define GEANY_HOMEPAGE					"http://www.geany.org/"
+#define GEANY_WIKI						"http://wiki.geany.org/"
+#define GEANY_BUG_REPORT				"http://www.geany.org/Support/Bugs"
+#define GEANY_DONATE					"https://sourceforge.net/donate/index.php?group_id=153444"
 #define GEANY_STRING_UNTITLED			_("untitled")
 #define GEANY_DEFAULT_DIALOG_HEIGHT		350
 #define GEANY_WINDOW_DEFAULT_WIDTH		900
@@ -70,7 +73,7 @@ typedef struct GeanyApp
 	 * This is a full path read by @ref tm_get_real_path().
 	 * @note Plugin configuration files should be saved as:
 	 * @code g_build_path(G_DIR_SEPARATOR_S, geany->app->configdir, "plugins", "pluginname",
-	 * 	"file.conf"); @endcode */
+	 * 	"file.conf", NULL); @endcode */
 	gchar				*configdir;
 	gchar				*datadir;
 	gchar				*docdir;
@@ -95,4 +98,6 @@ void geany_debug(gchar const *format, ...) G_GNUC_PRINTF (1, 2);
 #define G_GNUC_WARN_UNUSED_RESULT
 #endif
 
+G_END_DECLS
+
 #endif
