@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,18 +14,18 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: msgwindow.c 4959 2010-05-29 18:36:17Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-/*
+/**
+ * @file msgwindow.h
  * Message window functions (status, compiler, messages windows).
  * Also compiler error message parsing and grep file and line parsing.
- */
-
+ *
+ * @see GeanyMainWidgets::message_window_notebook to append a new notebook page.
+ **/
 
 #include "geany.h"
 
@@ -69,7 +69,6 @@ static void prepare_msg_tree_view(void);
 static void prepare_status_tree_view(void);
 static void prepare_compiler_tree_view(void);
 static GtkWidget *create_message_popup_menu(gint type);
-static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *line);
 static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
 																			gpointer user_data);
 static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data);
@@ -84,6 +83,16 @@ void msgwin_show_hide_tabs(void)
 }
 
 
+/** Sets the Messages path for opening any parsed filenames without absolute path
+ * from message lines.
+ * @param messages_dir The directory. **/
+void msgwin_set_messages_dir(const gchar *messages_dir)
+{
+	g_free(msgwindow.messages_dir);
+	msgwindow.messages_dir = g_strdup(messages_dir);
+}
+
+
 void msgwin_init(void)
 {
 	msgwindow.notebook = ui_lookup_widget(main_widgets.window, "notebook_info");
@@ -91,7 +100,7 @@ void msgwin_init(void)
 	msgwindow.tree_msg = ui_lookup_widget(main_widgets.window, "treeview4");
 	msgwindow.tree_compiler = ui_lookup_widget(main_widgets.window, "treeview5");
 	msgwindow.scribble = ui_lookup_widget(main_widgets.window, "textview_scribble");
-	msgwindow.find_in_files_dir = NULL;
+	msgwindow.messages_dir = NULL;
 
 	prepare_status_tree_view();
 	prepare_msg_tree_view();
@@ -107,24 +116,26 @@ void msgwin_init(void)
 
 void msgwin_finalize(void)
 {
-	g_free(msgwindow.find_in_files_dir);
+	g_free(msgwindow.messages_dir);
 }
 
 
 static gboolean on_msgwin_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
+	gboolean enter_or_return = ui_is_keyval_enter_or_return(event->keyval);
+
+	if (enter_or_return || event->keyval == GDK_space)
 	{
 		switch (GPOINTER_TO_INT(data))
 		{
 			case MSG_COMPILER:
-			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line(event->keyval);
+			{	/* key press in the compiler treeview */
+				msgwin_goto_compiler_file_line(enter_or_return);
 				break;
 			}
 			case MSG_MESSAGE:
-			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line(event->keyval);
+			{	/* key press in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(enter_or_return);
 				break;
 			}
 		}
@@ -183,6 +194,9 @@ static void prepare_msg_tree_view(void)
 	 * (connect_after button-press-event doesn't work) */
 	g_signal_connect(msgwindow.tree_msg, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
+	/* for double-clicking only, after the first release */
+	g_signal_connect(msgwindow.tree_msg, "button-press-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 	g_signal_connect(msgwindow.tree_msg, "key-press-event",
 		G_CALLBACK(on_msgwin_key_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 
@@ -216,6 +230,9 @@ static void prepare_compiler_tree_view(void)
 	 * (connect_after button-press-event doesn't work) */
 	g_signal_connect(msgwindow.tree_compiler, "button-release-event",
 					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
+	/* for double-clicking only, after the first release */
+	g_signal_connect(msgwindow.tree_compiler, "button-press-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
 	g_signal_connect(msgwindow.tree_compiler, "key-press-event",
 		G_CALLBACK(on_msgwin_key_press_event), GINT_TO_POINTER(MSG_COMPILER));
 
@@ -413,8 +430,7 @@ void msgwin_status_add(const gchar *format, ...)
 
 
 static void
-on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_message_treeview_clear_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	gint tabnum = GPOINTER_TO_INT(user_data);
 
@@ -423,8 +439,7 @@ on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
 
 
 static void
-on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_compiler_treeview_copy_activate(GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkWidget *tv = NULL;
 	GtkTreeSelection *selection;
@@ -455,7 +470,7 @@ on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
 		gchar *string;
 
 		gtk_tree_model_get(model, &iter, str_idx, &string, -1);
-		if (NZV(string))
+		if (!EMPTY(string))
 		{
 			gtk_clipboard_set_text(gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)),
 				string, -1);
@@ -497,7 +512,7 @@ static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpoint
 		gchar *line;
 
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, str_idx, &line, -1);
-		if (NZV(line))
+		if (!EMPTY(line))
 		{
 			g_string_append(str, line);
 			g_string_append_c(str, '\n');
@@ -520,8 +535,7 @@ static void on_compiler_treeview_copy_all_activate(GtkMenuItem *menuitem, gpoint
 
 
 static void
-on_hide_message_window                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+on_hide_message_window(GtkMenuItem *menuitem, gpointer user_data)
 {
 	msgwin_show_hide(FALSE);
 }
@@ -533,7 +547,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 
 	message_popup_menu = gtk_menu_new();
 
-	clear = gtk_image_menu_item_new_from_stock("gtk-clear", NULL);
+	clear = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR, NULL);
 	gtk_widget_show(clear);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), clear);
 	g_signal_connect(clear, "activate",
@@ -542,7 +556,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 	copy = gtk_image_menu_item_new_with_mnemonic(_("C_opy"));
 	gtk_widget_show(copy);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
-	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy), image);
 	g_signal_connect(copy, "activate",
@@ -551,7 +565,7 @@ static GtkWidget *create_message_popup_menu(gint type)
 	copy_all = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
 	gtk_widget_show(copy_all);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy_all);
-	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_stock(GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_all), image);
 	g_signal_connect(copy_all, "activate",
@@ -611,7 +625,7 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 }
 
 
-static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint keyval)
+static gboolean goto_compiler_file_line(const gchar *filename, gint line, gboolean focus_editor)
 {
 	if (!filename || line <= -1)
 		return FALSE;
@@ -627,9 +641,9 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint
 		if (cur_dir)
 		{
 			/* we let the user know we couldn't find the parsed filename from the message window */
-			setptr(cur_dir, utils_get_locale_from_utf8(cur_dir));
+			SETPTR(cur_dir, utils_get_locale_from_utf8(cur_dir));
 			name = g_path_get_basename(filename);
-			setptr(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
+			SETPTR(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
 			g_free(cur_dir);
 
 			if (g_file_test(name, G_FILE_TEST_EXISTS))
@@ -661,7 +675,7 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint
 				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
 
 			ret = navqueue_goto_line(old_doc, doc, line);
-			if (ret && ui_is_keyval_enter_or_return(keyval))
+			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 
 			return ret;
@@ -671,7 +685,7 @@ static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint
 }
 
 
-gboolean msgwin_goto_compiler_file_line(guint keyval)
+gboolean msgwin_goto_compiler_file_line(gboolean focus_editor)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -707,7 +721,7 @@ gboolean msgwin_goto_compiler_file_line(guint keyval)
 			g_free(string);
 			g_free(dir);
 
-			ret = goto_compiler_file_line(filename, line, keyval);
+			ret = goto_compiler_file_line(filename, line, focus_editor);
 			g_free(filename);
 			return ret;
 		}
@@ -730,8 +744,7 @@ static void make_absolute(gchar **filename, const gchar *dir)
 
 	/* add directory */
 	if (! utils_is_absolute_path(*filename))
-		setptr(*filename, g_strconcat(dir, G_DIR_SEPARATOR_S,
-			*filename + skip_dot_slash, NULL));
+		SETPTR(*filename, g_build_filename(dir, *filename + skip_dot_slash, NULL));
 }
 
 
@@ -830,11 +843,25 @@ static void parse_compiler_error_line(const gchar *string,
 		case GEANY_FILETYPES_PYTHON:
 		{
 			/* File "HyperArch.py", line 37, in ?
-			 * (file "clrdial.tcl" line 12) */
-			data.pattern = " \"";
-			data.min_fields = 6;
-			data.line_idx = 5;
-			data.file_idx = 2;
+			 * (file "clrdial.tcl" line 12)
+			 * */
+			if (strstr(string, " line ") != NULL)
+			{
+				/* Tcl and old Python format (<= Python 2.5) */
+				data.pattern = " \"";
+				data.min_fields = 6;
+				data.line_idx = 5;
+				data.file_idx = 2;
+			}
+			else
+			{
+				/* SyntaxError: ('invalid syntax', ('sender.py', 149, 20, ' ...'))
+				 * (used since Python 2.6) */
+				data.pattern = ",'";
+				data.min_fields = 8;
+				data.line_idx = 6;
+				data.file_idx = 4;
+			}
 			break;
 		}
 		case GEANY_FILETYPES_BASIC:
@@ -984,7 +1011,52 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 }
 
 
-gboolean msgwin_goto_messages_file_line(guint keyval)
+/* Tries to parse strings of the file:line style, allowing line field to be missing
+ * * filename is filled with the filename, should be freed
+ * * line is filled with the line number or -1 */
+static void msgwin_parse_generic_line(const gchar *string, gchar **filename, gint *line)
+{
+	gchar **fields;
+	gboolean incertain = TRUE; /* whether we're reasonably certain of the result */
+
+	*filename = NULL;
+	*line = -1;
+
+	fields = g_strsplit(string, ":", 2);
+	/* extract the filename */
+	if (fields[0] != NULL)
+	{
+		*filename = g_strdup(fields[0]);
+		if (msgwindow.messages_dir != NULL)
+			make_absolute(filename, msgwindow.messages_dir);
+
+		/* now the line */
+		if (fields[1] != NULL)
+		{
+			gchar *end;
+
+			*line = strtol(fields[1], &end, 10);
+			if (end == fields[1])
+				*line = -1;
+			else if (*end == ':' || g_ascii_isspace(*end))
+			{	/* if we have a blank or a separator right after the number, assume we really got a
+				 * filename (it's a grep-like syntax) */
+				incertain = FALSE;
+			}
+		}
+
+		/* if we aren't sure we got a supposedly correct filename, check it */
+		if (incertain && ! g_file_test(*filename, G_FILE_TEST_EXISTS))
+		{
+			SETPTR(*filename, NULL);
+			*line = -1;
+		}
+	}
+	g_strfreev(fields);
+}
+
+
+gboolean msgwin_goto_messages_file_line(gboolean focus_editor)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -1004,21 +1076,23 @@ gboolean msgwin_goto_messages_file_line(guint keyval)
 		if (line >= 0 && DOC_VALID(doc))
 		{
 			ret = navqueue_goto_line(old_doc, doc, line);
-			if (ret && ui_is_keyval_enter_or_return(keyval))
+			if (ret && focus_editor)
 				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 		}
 		else if (line < 0 && string != NULL)
 		{
 			gchar *filename;
-			msgwin_parse_grep_line(string, &filename, &line);
-			if (filename != NULL && line > -1)
+
+			/* try with a file:line parsing */
+			msgwin_parse_generic_line(string, &filename, &line);
+			if (filename != NULL)
 			{
 				/* use document_open_file to find an already open file, or open it in place */
 				doc = document_open_file(filename, FALSE, NULL, NULL);
 				if (doc != NULL)
 				{
-					ret = navqueue_goto_line(old_doc, doc, line);
-					if (ret && ui_is_keyval_enter_or_return(keyval))
+					ret = (line < 0) ? TRUE : navqueue_goto_line(old_doc, doc, line);
+					if (ret && focus_editor)
 						gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 				}
 			}
@@ -1030,53 +1104,28 @@ gboolean msgwin_goto_messages_file_line(guint keyval)
 }
 
 
-/* Try to parse the file and line number for string and when something useful is
- * found, store the line number in *line and the relevant file with the error in
- * *filename.
- * *line will be -1 if no error was found in string.
- * *filename must be freed unless NULL. */
-static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *line)
-{
-	ParseData data;
-
-	*filename = NULL;
-	*line = -1;
-
-	if (string == NULL || msgwindow.find_in_files_dir == NULL)
-		return;
-
-	/* conflict:3:conflicting types for `foo' */
-	data.string = string;
-	data.pattern = ":";
-	data.min_fields = 3;
-	data.line_idx = 1;
-	data.file_idx = 0;
-
-	parse_file_line(&data, filename, line);
-	make_absolute(filename, msgwindow.find_in_files_dir);
-}
-
-
 static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
 											 gpointer user_data)
 {
 	/* user_data might be NULL, GPOINTER_TO_INT returns 0 if called with NULL */
+	gboolean double_click = event->type == GDK_2BUTTON_PRESS;
 
-	if (event->button == 1)
+	if (event->button == 1 && (event->type == GDK_BUTTON_RELEASE || double_click))
 	{
 		switch (GPOINTER_TO_INT(user_data))
 		{
 			case MSG_COMPILER:
-			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line(0);
+			{	/* mouse click in the compiler treeview */
+				msgwin_goto_compiler_file_line(double_click);
 				break;
 			}
 			case MSG_MESSAGE:
-			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line(0);
+			{	/* mouse click in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(double_click);
 				break;
 			}
 		}
+		return double_click;	/* TRUE prevents message window re-focusing */
 	}
 
 	if (event->button == 3)
@@ -1173,5 +1222,3 @@ void msgwin_clear_tab(gint tabnum)
 		return;
 	gtk_list_store_clear(store);
 }
-
-
