@@ -1,8 +1,8 @@
 /*
  *      notebook.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- * $Id: notebook.c 4928 2010-05-16 17:45:39Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -26,6 +24,9 @@
  */
 
 #include "geany.h"
+
+#include <gdk/gdkkeysyms.h>
+
 #include "notebook.h"
 #include "document.h"
 #include "editor.h"
@@ -36,6 +37,8 @@
 #include "callbacks.h"
 #include "utils.h"
 #include "keybindings.h"
+#include "main.h"
+#include "gtkcompat.h"
 
 #define GEANY_DND_NOTEBOOK_TAB_TYPE	"geany_dnd_notebook_tab"
 
@@ -51,22 +54,23 @@ static GtkTargetEntry files_drop_targets[] = {
 	{ "text/uri-list",	0, 0 }
 };
 
+static const gsize MAX_MRU_DOCS = 20;
+static GQueue *mru_docs = NULL;
+static guint mru_pos = 0;
+
+static gboolean switch_in_progress = FALSE;
+static GtkWidget *switch_dialog = NULL;
+static GtkWidget *switch_dialog_label = NULL;
 
-static gboolean
-notebook_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
-	gint x, gint y, guint event_time, gpointer user_data);
 
 static void
 notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_num,
-	gpointer user_data);
+		gpointer user_data);
 
 static void
 on_window_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
-                             gint x, gint y, GtkSelectionData *data, guint info,
-                             guint event_time, gpointer user_data);
-
-static gint
-notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y);
+		gint x, gint y, GtkSelectionData *data, guint target_type,
+		guint event_time, gpointer user_data);
 
 static void
 notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data);
@@ -74,6 +78,228 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data);
 static void setup_tab_dnd(void);
 
 
+static void update_mru_docs_head(GeanyDocument *doc)
+{
+	if (doc)
+	{
+		g_queue_remove(mru_docs, doc);
+		g_queue_push_head(mru_docs, doc);
+
+		if (g_queue_get_length(mru_docs) > MAX_MRU_DOCS)
+			g_queue_pop_tail(mru_docs);
+	}
+}
+
+
+/* before the tab changes, add the current document to the MRU list */
+static void on_notebook_switch_page(GtkNotebook *notebook,
+	gpointer page, guint page_num, gpointer user_data)
+{
+	GeanyDocument *new;
+
+	new = document_get_from_page(page_num);
+
+	/* insert the very first document (when adding the second document
+	 * and switching to it) */
+	if (g_queue_get_length(mru_docs) == 0 && gtk_notebook_get_n_pages(notebook) == 2)
+		update_mru_docs_head(document_get_current());
+
+	if (!switch_in_progress)
+		update_mru_docs_head(new);
+}
+
+
+static void on_document_close(GObject *obj, GeanyDocument *doc)
+{
+	if (! main_status.quitting)
+	{
+		g_queue_remove(mru_docs, doc);
+		/* this prevents the pop up window from showing when there's a single
+		 * document */
+		if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook)) == 2)
+			g_queue_clear(mru_docs);
+	}
+}
+
+
+static GtkWidget *ui_minimal_dialog_new(GtkWindow *parent, const gchar *title)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_window_new(GTK_WINDOW_POPUP);
+
+	if (parent)
+	{
+		gtk_window_set_transient_for(GTK_WINDOW(dialog), parent);
+		gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
+	}
+	gtk_window_set_title(GTK_WINDOW(dialog), title);
+	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
+	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
+
+	gtk_widget_set_name(dialog, "GeanyDialog");
+	return dialog;
+}
+
+
+static gboolean is_modifier_key(guint keyval)
+{
+	switch (keyval)
+	{
+		case GDK_Shift_L:
+		case GDK_Shift_R:
+		case GDK_Control_L:
+		case GDK_Control_R:
+		case GDK_Meta_L:
+		case GDK_Meta_R:
+		case GDK_Alt_L:
+		case GDK_Alt_R:
+		case GDK_Super_L:
+		case GDK_Super_R:
+		case GDK_Hyper_L:
+		case GDK_Hyper_R:
+			return TRUE;
+		default:
+			return FALSE;
+	}
+}
+
+
+static gboolean on_key_release_event(GtkWidget *widget, GdkEventKey *ev, gpointer user_data)
+{
+	/* user may have rebound keybinding to a different modifier than Ctrl, so check all */
+	if (switch_in_progress && is_modifier_key(ev->keyval))
+	{
+		GeanyDocument *doc;
+
+		switch_in_progress = FALSE;
+
+		if (switch_dialog)
+		{
+			gtk_widget_destroy(switch_dialog);
+			switch_dialog = NULL;
+		}
+
+		doc = document_get_current();
+		update_mru_docs_head(doc);
+		mru_pos = 0;
+		document_check_disk_status(doc, TRUE);
+	}
+	return FALSE;
+}
+
+
+static GtkWidget *create_switch_dialog(void)
+{
+	GtkWidget *dialog, *widget, *vbox;
+
+	dialog = ui_minimal_dialog_new(GTK_WINDOW(main_widgets.window), _("Switch to Document"));
+	gtk_window_set_decorated(GTK_WINDOW(dialog), FALSE);
+	gtk_window_set_default_size(GTK_WINDOW(dialog), 200, -1);
+
+	vbox = gtk_vbox_new(FALSE, 6);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
+	gtk_container_add(GTK_CONTAINER(dialog), vbox);
+
+	widget = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_BUTTON);
+	gtk_container_add(GTK_CONTAINER(vbox), widget);
+
+	widget = gtk_label_new(NULL);
+	gtk_label_set_justify(GTK_LABEL(widget), GTK_JUSTIFY_CENTER);
+	gtk_container_add(GTK_CONTAINER(vbox), widget);
+	switch_dialog_label = widget;
+
+	g_signal_connect(dialog, "key-release-event", G_CALLBACK(on_key_release_event), NULL);
+	return dialog;
+}
+
+
+static void update_filename_label(void)
+{
+	guint i;
+	gchar *msg = NULL;
+	guint queue_length;
+	GeanyDocument *doc;
+
+	if (!switch_dialog)
+	{
+		switch_dialog = create_switch_dialog();
+		gtk_widget_show_all(switch_dialog);
+	}
+
+	queue_length = g_queue_get_length(mru_docs);
+	for (i = mru_pos; (i <= mru_pos + 3) && (doc = g_queue_peek_nth(mru_docs, i % queue_length)); i++)
+	{
+		gchar *basename;
+
+		basename = g_path_get_basename(DOC_FILENAME(doc));
+		if (i == mru_pos)
+			msg = g_markup_printf_escaped ("<b>%s</b>", basename);
+		else if (i % queue_length == mru_pos)    /* && i != mru_pos */
+		{
+			/* We have wrapped around and got to the starting document again */
+			g_free(basename);
+			break;
+		}
+		else
+		{
+			SETPTR(basename, g_markup_printf_escaped ("\n%s", basename));
+			SETPTR(msg, g_strconcat(msg, basename, NULL));
+		}
+		g_free(basename);
+	}
+	gtk_label_set_markup(GTK_LABEL(switch_dialog_label), msg);
+	g_free(msg);
+}
+
+
+static gboolean on_switch_timeout(G_GNUC_UNUSED gpointer data)
+{
+	if (!switch_in_progress || switch_dialog)
+	{
+		return FALSE;
+	}
+
+	update_filename_label();
+	return FALSE;
+}
+
+
+void notebook_switch_tablastused(void)
+{
+	GeanyDocument *last_doc;
+	gboolean switch_start = !switch_in_progress;
+
+	mru_pos += 1;
+	last_doc = g_queue_peek_nth(mru_docs, mru_pos);
+
+	if (! DOC_VALID(last_doc))
+	{
+		utils_beep();
+		mru_pos = 0;
+		last_doc = g_queue_peek_nth(mru_docs, mru_pos);
+	}
+	if (! DOC_VALID(last_doc))
+		return;
+
+	switch_in_progress = TRUE;
+	document_show_tab(last_doc);
+
+	/* if there's a modifier key, we can switch back in MRU order each time unless
+	 * the key is released */
+	if (switch_start)
+		g_timeout_add(600, on_switch_timeout, NULL);
+	else
+		update_filename_label();
+}
+
+
+gboolean notebook_switch_in_progress(void)
+{
+	return switch_in_progress;
+}
+
+
 static gboolean focus_sci(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
 	GeanyDocument *doc = document_get_current();
@@ -87,7 +313,7 @@ static gboolean focus_sci(GtkWidget *widget, GdkEventButton *event, gpointer use
 
 static gboolean gtk_notebook_show_arrows(GtkNotebook *notebook)
 {
-	return notebook->scrollable;
+	return gtk_notebook_get_scrollable(notebook);
 #if 0
 	/* To get this working we would need to define at least the first two fields of
 	 * GtkNotebookPage since it is a private field. The better way would be to
@@ -137,12 +363,8 @@ static gboolean is_position_on_tab_bar(GtkNotebook *notebook, GdkEventButton *ev
 	tab_pos = gtk_notebook_get_tab_pos(notebook);
 	nb = GTK_WIDGET(notebook);
 
-#if GTK_CHECK_VERSION(2, 10, 0)
 	gtk_widget_style_get(GTK_WIDGET(notebook), "scroll-arrow-hlength", &scroll_arrow_hlength,
 		"scroll-arrow-vlength", &scroll_arrow_vlength, NULL);
-#else
-	scroll_arrow_hlength = scroll_arrow_vlength = 16;
-#endif
 
 	if (! gdk_event_get_coords((GdkEvent*) event, &x, &y))
 	{
@@ -155,11 +377,11 @@ static gboolean is_position_on_tab_bar(GtkNotebook *notebook, GdkEventButton *ev
 		case GTK_POS_TOP:
 		case GTK_POS_BOTTOM:
 		{
-			if (event->y >= 0 && event->y <= tab->allocation.height)
+			if (event->y >= 0 && event->y <= gtk_widget_get_allocated_height(tab))
 			{
 				if (! gtk_notebook_show_arrows(notebook) || (
 					x > scroll_arrow_hlength &&
-					x < nb->allocation.width - scroll_arrow_hlength))
+					x < gtk_widget_get_allocated_width(nb) - scroll_arrow_hlength))
 					return TRUE;
 			}
 			break;
@@ -167,11 +389,11 @@ static gboolean is_position_on_tab_bar(GtkNotebook *notebook, GdkEventButton *ev
 		case GTK_POS_LEFT:
 		case GTK_POS_RIGHT:
 		{
-			if (event->x >= 0 && event->x <= tab->allocation.width)
+			if (event->x >= 0 && event->x <= gtk_widget_get_allocated_width(tab))
 			{
 				if (! gtk_notebook_show_arrows(notebook) || (
 					y > scroll_arrow_vlength &&
-					y < nb->allocation.height - scroll_arrow_vlength))
+					y < gtk_widget_get_allocated_height(nb) - scroll_arrow_vlength))
 					return TRUE;
 			}
 		}
@@ -188,15 +410,44 @@ static void tab_bar_menu_activate_cb(GtkMenuItem *menuitem, gpointer data)
 	if (! DOC_VALID(doc))
 		return;
 
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-		document_get_notebook_page(doc));
+	document_show_tab(doc);
 }
 
 
-static void show_tab_bar_popup_menu(GdkEventButton *event)
+static void on_open_in_new_window_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+	gchar *geany_path;
+	GeanyDocument *doc = user_data;
+
+	g_return_if_fail(doc->is_valid);
+
+	geany_path = g_find_program_in_path("geany");
+
+	if (geany_path)
+	{
+		gchar *doc_path = utils_get_locale_from_utf8(doc->file_name);
+		gchar *argv[] = {geany_path, "-i", doc_path, NULL};
+		GError *err = NULL;
+
+		if (!utils_spawn_async(NULL, argv, NULL, 0, NULL, NULL, NULL, &err))
+		{
+			g_printerr("Unable to open new window: %s", err->message);
+			g_error_free(err);
+		}
+		g_free(doc_path);
+		g_free(geany_path);
+	}
+	else
+		g_printerr("Unable to find 'geany'");
+}
+
+
+static void show_tab_bar_popup_menu(GdkEventButton *event, GtkWidget *page)
 {
 	GtkWidget *menu_item;
 	static GtkWidget *menu = NULL;
+	GeanyDocument *doc = NULL;
+	gint page_num;
 
 	if (menu == NULL)
 		menu = gtk_menu_new();
@@ -211,10 +462,36 @@ static void show_tab_bar_popup_menu(GdkEventButton *event)
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
 
+	if (page != NULL)
+	{
+		page_num = gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook), page);
+		doc = document_get_from_page(page_num);
+	}
+
+	menu_item = ui_image_menu_item_new(GTK_STOCK_OPEN, "Open in New _Window");
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+	g_signal_connect(menu_item, "activate",
+		G_CALLBACK(on_open_in_new_window_activate), doc);
+	/* disable if not on disk */
+	if (doc == NULL || !doc->real_path)
+		gtk_widget_set_sensitive(menu_item, FALSE);
+
+	menu_item = gtk_separator_menu_item_new();
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+
+	menu_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
+	gtk_widget_show(menu_item);
+	gtk_container_add(GTK_CONTAINER(menu), menu_item);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(notebook_tab_close_clicked_cb), page);
+	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), (page != NULL));
+
 	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("Close Ot_her Documents"));
 	gtk_widget_show(menu_item);
 	gtk_container_add(GTK_CONTAINER(menu), menu_item);
-	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), NULL);
+	g_signal_connect(menu_item, "activate", G_CALLBACK(on_close_other_documents1_activate), page);
+	gtk_widget_set_sensitive(GTK_WIDGET(menu_item), (page != NULL));
 
 	menu_item = ui_image_menu_item_new(GTK_STOCK_CLOSE, _("C_lose All"));
 	gtk_widget_show(menu_item);
@@ -230,74 +507,65 @@ static gboolean notebook_tab_bar_click_cb(GtkWidget *widget, GdkEventButton *eve
 {
 	if (event->type == GDK_2BUTTON_PRESS)
 	{
-		/* accessing ::event_window is a little hacky but we need to make sure the click
-		 * was in the tab bar and not inside the child */
-		if (event->window != GTK_NOTEBOOK(main_widgets.notebook)->event_window)
+		GtkNotebook *notebook = GTK_NOTEBOOK(widget);
+		GtkWidget *event_widget = gtk_get_event_widget((GdkEvent *) event);
+		GtkWidget *child = gtk_notebook_get_nth_page(notebook, gtk_notebook_get_current_page(notebook));
+
+		/* ignore events from the content of the page (impl. stolen from GTK2 tab scrolling)
+		 * TODO: we should also ignore notebook's action widgets, but that's more work and
+		 * we don't have any of them yet anyway -- and GTK 2.16 don't have those actions. */
+		if (event_widget == NULL || event_widget == child || gtk_widget_is_ancestor(event_widget, child))
 			return FALSE;
 
-		if (is_position_on_tab_bar(GTK_NOTEBOOK(widget), event))
+		if (is_position_on_tab_bar(notebook, event))
 		{
 			document_new_file(NULL, NULL, NULL);
 			return TRUE;
 		}
 	}
+	/* right-click is also handled here if it happened on the notebook tab bar but not
+	 * on a tab directly */
 	else if (event->button == 3)
 	{
-		show_tab_bar_popup_menu(event);
+		show_tab_bar_popup_menu(event, NULL);
+		return TRUE;
 	}
 	return FALSE;
 }
 
 
-void notebook_init()
+void notebook_init(void)
 {
-	/* Individual style for the tab close buttons */
-	gtk_rc_parse_string(
-		"style \"geany-close-tab-button-style\" {\n"
-		"	GtkWidget::focus-padding = 0\n"
-		"	GtkWidget::focus-line-width = 0\n"
-		"	xthickness = 0\n"
-		"	ythickness = 0\n"
-		"}\n"
-		"widget \"*.geany-close-tab-button\" style \"geany-close-tab-button-style\""
-	);
-
 	g_signal_connect_after(main_widgets.notebook, "button-press-event",
 		G_CALLBACK(notebook_tab_bar_click_cb), NULL);
 
 	g_signal_connect(main_widgets.notebook, "drag-data-received",
 		G_CALLBACK(on_window_drag_data_received), NULL);
 
+	mru_docs = g_queue_new();
+	g_signal_connect(main_widgets.notebook, "switch-page",
+		G_CALLBACK(on_notebook_switch_page), NULL);
+	g_signal_connect(geany_object, "document-close",
+		G_CALLBACK(on_document_close), NULL);
+
+	/* in case the switch dialog misses an event while drawing the dialog */
+	g_signal_connect(main_widgets.window, "key-release-event", G_CALLBACK(on_key_release_event), NULL);
+
 	setup_tab_dnd();
 }
 
 
-static void setup_tab_dnd()
+void notebook_free(void)
+{
+	g_queue_free(mru_docs);
+}
+
+
+static void setup_tab_dnd(void)
 {
 	GtkWidget *notebook = main_widgets.notebook;
 
-	/* Due to a segfault with manual tab DnD setup on GTK 2.10, we must
-	*  use the built in gtk_notebook_set_tab_reorderable from GTK 2.10.
-	*  This means a binary compiled against < 2.10 but run on >= 2.10
-	*  will not have tab DnD support, but this is necessary until
-	*  there is a fix for the older tab DnD code or GTK 2.10. */
-	if (gtk_check_version(2, 10, 0) == NULL) /* null means version ok */
-	{
-#if GTK_CHECK_VERSION(2, 10, 0)
-		g_signal_connect(notebook, "page-reordered", G_CALLBACK(notebook_page_reordered_cb), NULL);
-#endif
-		return;
-	}
-
-	/* Set up drag movement callback */
-	g_signal_connect(notebook, "drag-motion", G_CALLBACK(notebook_drag_motion_cb), NULL);
-
-	/* set up drag motion for moving notebook pages */
-	gtk_drag_dest_set(notebook, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
-		drag_targets, G_N_ELEMENTS(drag_targets), GDK_ACTION_MOVE);
-	/* set drag source, but for GTK+2.6 it's changed in motion-notify-event handler */
-	gtk_drag_source_set(notebook, GDK_BUTTON1_MASK,
-		drag_targets, G_N_ELEMENTS(drag_targets), GDK_ACTION_MOVE);
+	g_signal_connect(notebook, "page-reordered", G_CALLBACK(notebook_page_reordered_cb), NULL);
 }
 
 
@@ -311,91 +579,6 @@ notebook_page_reordered_cb(GtkNotebook *notebook, GtkWidget *child, guint page_n
 }
 
 
-static gboolean
-notebook_drag_motion_cb(GtkWidget *widget, GdkDragContext *dc,
-	gint x, gint y, guint event_time, gpointer user_data)
-{
-	static gint oldx, oldy; /* for determining direction of mouse drag */
-	GtkNotebook *notebook = GTK_NOTEBOOK(widget);
-	gint ndest = notebook_find_tab_num_at_pos(notebook, x, y);
-	gint ncurr = gtk_notebook_get_current_page(notebook);
-
-	if (ndest >= 0 && ndest != ncurr)
-	{
-		gboolean ok = FALSE;
-		/* prevent oscillation between non-homogeneous sized tabs */
-		switch (gtk_notebook_get_tab_pos(notebook))
-		{
-			case GTK_POS_LEFT:
-			case GTK_POS_RIGHT:
-			ok = ((ndest > ncurr) && (y > oldy)) || ((ndest < ncurr) && (y < oldy));
-			break;
-
-			case GTK_POS_TOP:
-			case GTK_POS_BOTTOM:
-			ok = ((ndest > ncurr) && (x > oldx)) || ((ndest < ncurr) && (x < oldx));
-			break;
-		}
-
-		if (ok)
-		{
-			gtk_notebook_reorder_child(notebook,
-							gtk_notebook_get_nth_page(notebook, ncurr), ndest);
-			notebook_page_reordered_cb(NULL, NULL, ndest, NULL);
-		}
-	}
-
-	oldx = x; oldy = y;
-	return FALSE;
-}
-
-
-/* Adapted from Epiphany absolute version in ephy-notebook.c, thanks.
- * x,y are co-ordinates local to the notebook (not including border padding)
- * notebook tab label widgets must not be NULL.
- * N.B. This only checks the dimension that the tabs are in,
- * e.g. for GTK_POS_TOP it does not check the y coordinate. */
-static gint
-notebook_find_tab_num_at_pos(GtkNotebook *notebook, gint x, gint y)
-{
-	GtkPositionType tab_pos;
-	int page_num = 0;
-	GtkWidget *page;
-
-	/* deal with less than 2 pages */
-	switch (gtk_notebook_get_n_pages(notebook))
-	{case 0: return -1; case 1: return 0;}
-
-	tab_pos = gtk_notebook_get_tab_pos(notebook); /* which edge */
-
-	while ((page = gtk_notebook_get_nth_page(notebook, page_num)))
-	{
-		gint max_x, max_y;
-		GtkWidget *tab = gtk_notebook_get_tab_label(notebook, page);
-
-		g_return_val_if_fail(tab != NULL, -1);
-
-		if (!GTK_WIDGET_MAPPED(GTK_WIDGET(tab)))
-		{ /* skip hidden tabs, e.g. tabs scrolled out of view */
-			page_num++;
-			continue;
-		}
-
-		/* subtract notebook pos to remove possible border padding */
-		max_x = tab->allocation.x + tab->allocation.width - GTK_WIDGET(notebook)->allocation.x;
-		max_y = tab->allocation.y + tab->allocation.height - GTK_WIDGET(notebook)->allocation.y;
-
-		if (((tab_pos == GTK_POS_TOP) || (tab_pos == GTK_POS_BOTTOM)) && (x <= max_x))
-			return page_num;
-		else if (((tab_pos == GTK_POS_LEFT) || (tab_pos == GTK_POS_RIGHT)) && (y <= max_y))
-			return page_num;
-
-		page_num++;
-	}
-	return -1;
-}
-
-
 /* call this after the number of tabs in main_widgets.notebook changes. */
 static void tab_count_changed(void)
 {
@@ -446,6 +629,13 @@ static gboolean notebook_tab_click(GtkWidget *widget, GdkEventButton *event, gpo
 			GEANY_KEYS_NOTEBOOK_SWITCHTABLASTUSED);
 		return TRUE;
 	}
+	/* right-click is first handled here if it happened on a notebook tab */
+	if (event->button == 3)
+	{
+		show_tab_bar_popup_menu(event, data);
+		return TRUE;
+	}
+
 	return FALSE;
 }
 
@@ -477,7 +667,7 @@ gint notebook_new_tab(GeanyDocument *this)
 	/* get button press events for the tab label and the space between it and
 	 * the close button, if any */
 	ebox = gtk_event_box_new();
-	GTK_WIDGET_SET_FLAGS(ebox, GTK_NO_WINDOW);
+	gtk_widget_set_has_window(ebox, FALSE);
 	g_signal_connect(ebox, "button-press-event", G_CALLBACK(notebook_tab_click), page);
 	/* focus the current document after clicking on a tab */
 	g_signal_connect_after(ebox, "button-release-event",
@@ -499,7 +689,7 @@ gint notebook_new_tab(GeanyDocument *this)
 		image = gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
 		gtk_container_add(GTK_CONTAINER(btn), image);
 
-		align = gtk_alignment_new(1.0, 0.0, 0.0, 0.0);
+		align = gtk_alignment_new(1.0, 0.5, 0.0, 0.0);
 		gtk_container_add(GTK_CONTAINER(align), btn);
 		gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 0);
 
@@ -527,13 +717,9 @@ gint notebook_new_tab(GeanyDocument *this)
 
 	tab_count_changed();
 
-	/* This is where tab DnD is enabled for GTK 2.10 and higher */
-#if GTK_CHECK_VERSION(2, 10, 0)
-	if (gtk_check_version(2, 10, 0) == NULL) /* null means version ok */
-	{
-		gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(main_widgets.notebook), page, TRUE);
-	}
-#endif
+	/* enable tab DnD */
+	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(main_widgets.notebook), page, TRUE);
+
 	return tabnum;
 }
 
@@ -551,12 +737,25 @@ notebook_tab_close_clicked_cb(GtkButton *button, gpointer user_data)
 /* Always use this instead of gtk_notebook_remove_page(). */
 void notebook_remove_page(gint page_num)
 {
-	gint curpage = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
+	gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(main_widgets.notebook));
 
-	/* Focus the next page, not the previous */
-	if (curpage == page_num && file_prefs.tab_order_ltr)
+	if (page_num == page)
 	{
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), curpage + 1);
+		if (file_prefs.tab_order_ltr)
+			page += 1;
+		else if (page > 0) /* never go negative, it would select the last page */
+			page -= 1;
+
+		if (file_prefs.tab_close_switch_to_mru)
+		{
+			GeanyDocument *last_doc;
+
+			last_doc = g_queue_peek_nth(mru_docs, 0);
+			if (DOC_VALID(last_doc))
+				page = document_get_notebook_page(last_doc);
+		}
+
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook), page);
 	}
 
 	/* now remove the page (so we don't temporarily switch to the previous page) */
@@ -568,23 +767,17 @@ void notebook_remove_page(gint page_num)
 
 static void
 on_window_drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
-                             gint x, gint y, GtkSelectionData *data, guint target_type,
-                             guint event_time, gpointer user_data)
+		gint x, gint y, GtkSelectionData *data, guint target_type,
+		guint event_time, gpointer user_data)
 {
 	gboolean success = FALSE;
+	gint length = gtk_selection_data_get_length(data);
 
-	if (data->length > 0 && data->format == 8)
+	if (length > 0 && gtk_selection_data_get_format(data) == 8)
 	{
-		if (drag_context->action == GDK_ACTION_ASK)
-		{
-			drag_context->action = GDK_ACTION_COPY;
-		}
-
-		document_open_file_list((const gchar *)data->data, data->length);
+		document_open_file_list((const gchar *)gtk_selection_data_get_data(data), length);
 
 		success = TRUE;
 	}
 	gtk_drag_finish(drag_context, success, FALSE, event_time);
 }
-
-
