@@ -1,8 +1,8 @@
 /*
  *      plugindata.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,12 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *      MA 02110-1301, USA.
- *
- * $Id: plugindata.h 5497 2010-12-22 16:59:38Z ntrel $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -32,8 +29,10 @@
  * when making changes (see 'Keeping the plugin ABI stable' in the HACKING file). */
 
 
-#ifndef PLUGINDATA_H
-#define PLUGINDATA_H
+#ifndef GEANY_PLUGINDATA_H
+#define GEANY_PLUGINDATA_H
+
+G_BEGIN_DECLS
 
 /* Compatibility for sharing macros between API and core.
  * First include geany.h, then plugindata.h, then other API headers. */
@@ -41,6 +40,8 @@
 #define GEANY(symbol_name) geany->symbol_name
 
 #include "editor.h"	/* GeanyIndentType */
+#include "build.h"  /* GeanyBuildGroup, GeanyBuildSource, GeanyBuildCmdEntries enums */
+#include "gtkcompat.h"
 
 
 /** The Application Programming Interface (API) version, incremented
@@ -54,14 +55,21 @@
  * @warning You should not test for values below 200 as previously
  * @c GEANY_API_VERSION was defined as an enum value, not a macro.
  */
-#define GEANY_API_VERSION 201
+#define GEANY_API_VERSION 217
 
+/* hack to have a different ABI when built with GTK3 because loading GTK2-linked plugins
+ * with GTK3-linked Geany leads to crash */
+#if GTK_CHECK_VERSION(3, 0, 0)
+#	define GEANY_ABI_SHIFT 8
+#else
+#	define GEANY_ABI_SHIFT 0
+#endif
 /** The Application Binary Interface (ABI) version, incremented whenever
  * existing fields in the plugin data types have to be changed or reordered.
  * Changing this forces all plugins to be recompiled before Geany can load them. */
 /* This should usually stay the same if fields are only appended, assuming only pointers to
  * structs and not structs themselves are declared by plugins. */
-#define GEANY_ABI_VERSION 68
+#define GEANY_ABI_VERSION (69 << GEANY_ABI_SHIFT)
 
 
 /** Defines a function to check the plugin is safe to load.
@@ -264,18 +272,19 @@ typedef struct GeanyFunctions
 	struct MsgWinFuncs			*p_msgwindow;
 	struct EncodingFuncs		*p_encodings;		/**< See encodings.h */
 	struct KeybindingFuncs		*p_keybindings;		/**< See keybindings.h */
-	struct TagManagerFuncs		*p_tm;				/**< See tagmanager/include */
+	struct TagManagerFuncs		*p_tm;				/**< See tagmanager/src */
 	struct SearchFuncs			*p_search;			/**< See search.h */
 	struct HighlightingFuncs	*p_highlighting;	/**< See highlighting.h */
 	struct FiletypeFuncs		*p_filetypes;		/**< See filetypes.h */
-	struct NavQueueFuncs        *p_navqueue;		/**< See navqueue.h */
-	struct EditorFuncs        	*p_editor;			/**< See editor.h */
-	struct MainFuncs        	*p_main;			/**< See main.h */
-	struct PluginFuncs        	*p_plugin;			/**< See pluginutils.c */
+	struct NavQueueFuncs		*p_navqueue;		/**< See navqueue.h */
+	struct EditorFuncs			*p_editor;			/**< See editor.h */
+	struct MainFuncs			*p_main;			/**< See main.h */
+	struct PluginFuncs			*p_plugin;			/**< See pluginutils.c */
 	struct ScintillaFuncs		*p_scintilla;		/**< See ScintillaFuncs */
 	struct MsgWinFuncs			*p_msgwin;			/**< See msgwindow.h */
 	struct StashFuncs			*p_stash;			/**< See stash.h */
 	struct SymbolsFuncs			*p_symbols;			/**< See symbols.h */
+	struct BuildFuncs			*p_build;			/**< See build.h */
 }
 GeanyFunctions;
 
@@ -310,6 +319,9 @@ typedef struct DocumentFuncs
 	const GdkColor*	(*document_get_status_color) (struct GeanyDocument *doc);
 	gchar*		(*document_get_basename_for_display) (struct GeanyDocument *doc, gint length);
 	gint		(*document_get_notebook_page) (struct GeanyDocument *doc);
+	gint		(*document_compare_by_display_name) (gconstpointer a, gconstpointer b);
+	gint		(*document_compare_by_tab_order) (gconstpointer a, gconstpointer b);
+	gint		(*document_compare_by_tab_order_reverse) (gconstpointer a, gconstpointer b);
 }
 DocumentFuncs;
 
@@ -436,6 +448,7 @@ typedef struct UtilsFuncs
 				GError **error);
 	gchar**		(*utils_copy_environment)(const gchar **exclude_vars, const gchar *first_varname, ...);
 	gchar*		(*utils_find_open_xml_tag) (const gchar sel[], gint size);
+	const gchar*	(*utils_find_open_xml_tag_pos) (const gchar sel[], gint size);
 }
 UtilsFuncs;
 
@@ -471,8 +484,11 @@ typedef struct UIUtilsFuncs
 	void		(*ui_widget_modify_font_from_string) (GtkWidget *widget, const gchar *str);
 	gboolean	(*ui_is_keyval_enter_or_return) (guint keyval);
 	gint		(*ui_get_gtk_settings_integer) (const gchar *property_name, gint default_value);
-	void		(*ui_combo_box_add_to_history) (GtkComboBoxEntry *combo_entry,
+	void		(*ui_combo_box_add_to_history) (GtkComboBoxText *combo_entry,
 				const gchar *text, gint history_len);
+	void		(*ui_menu_add_document_items_sorted) (GtkMenu *menu, struct GeanyDocument *active,
+				GCallback callback, GCompareFunc compare_func);
+	const gchar* (*ui_lookup_stock_label)(const gchar *stock_id);
 }
 UIUtilsFuncs;
 
@@ -517,8 +533,8 @@ MsgWinFuncs;
 /* See encodings.h */
 typedef struct EncodingFuncs
 {
-	gchar*			(*encodings_convert_to_utf8) (const gchar *buffer, gsize size, gchar **used_encoding);
-	gchar* 			(*encodings_convert_to_utf8_from_charset) (const gchar *buffer, gsize size,
+	gchar*			(*encodings_convert_to_utf8) (const gchar *buffer, gssize size, gchar **used_encoding);
+	gchar* 			(*encodings_convert_to_utf8_from_charset) (const gchar *buffer, gssize size,
 													 const gchar *charset, gboolean fast);
 	const gchar*	(*encodings_get_charset_from_index) (gint idx);
 }
@@ -561,6 +577,7 @@ typedef struct FiletypeFuncs
 	GeanyFiletype*	(*filetypes_lookup_by_name) (const gchar *name);
 	GeanyFiletype*	(*filetypes_index)(gint idx);
 	const gchar*	(*filetypes_get_display_name)(GeanyFiletype *ft);
+	const GSList*	(*filetypes_get_sorted_by_name)(void);
 	/* Remember to convert any filetype_id arguments to GeanyFiletype pointers in any
 	 * appended functions */
 }
@@ -644,6 +661,12 @@ typedef struct PluginFuncs
 	struct GeanyKeyGroup* (*plugin_set_key_group)(GeanyPlugin *plugin,
 		const gchar *section_name, gsize count, _GeanyKeyGroupCallback callback);
 	void	(*plugin_show_configure)(GeanyPlugin *plugin);
+	guint	(*plugin_timeout_add) (GeanyPlugin *plugin, guint interval, GSourceFunc function,
+		gpointer data);
+	guint	(*plugin_timeout_add_seconds) (GeanyPlugin *plugin, guint interval,
+		GSourceFunc function, gpointer data);
+	guint	(*plugin_idle_add) (GeanyPlugin *plugin, GSourceFunc function, gpointer data);
+	void	(*plugin_builder_connect_signals) (GeanyPlugin *plugin, GtkBuilder *builder, gpointer user_data);
 }
 PluginFuncs;
 
@@ -669,23 +692,24 @@ typedef struct StashFuncs
 	gint (*stash_group_save_to_file)(struct StashGroup *group, const gchar *filename,
 			GKeyFileFlags flags);
 	void (*stash_group_add_toggle_button)(struct StashGroup *group, gboolean *setting,
-			const gchar *key_name, gboolean default_value, gpointer widget_id);
+			const gchar *key_name, gboolean default_value, gconstpointer widget_id);
 	void (*stash_group_add_radio_buttons)(struct StashGroup *group, gint *setting,
 			const gchar *key_name, gint default_value,
-			gpointer widget_id, gint enum_id, ...) G_GNUC_NULL_TERMINATED;
+			gconstpointer widget_id, gint enum_id, ...) G_GNUC_NULL_TERMINATED;
 	void (*stash_group_add_spin_button_integer)(struct StashGroup *group, gint *setting,
-			const gchar *key_name, gint default_value, gpointer widget_id);
+			const gchar *key_name, gint default_value, gconstpointer widget_id);
 	void (*stash_group_add_combo_box)(struct StashGroup *group, gint *setting,
-			const gchar *key_name, gint default_value, gpointer widget_id);
+			const gchar *key_name, gint default_value, gconstpointer widget_id);
 	void (*stash_group_add_combo_box_entry)(struct StashGroup *group, gchar **setting,
-			const gchar *key_name, const gchar *default_value, gpointer widget_id);
+			const gchar *key_name, const gchar *default_value, gconstpointer widget_id);
 	void (*stash_group_add_entry)(struct StashGroup *group, gchar **setting,
-			const gchar *key_name, const gchar *default_value, gpointer widget_id);
+			const gchar *key_name, const gchar *default_value, gconstpointer widget_id);
 	void (*stash_group_add_widget_property)(struct StashGroup *group, gpointer setting,
-			const gchar *key_name, gpointer default_value, gpointer widget_id,
+			const gchar *key_name, gpointer default_value, gconstpointer widget_id,
 			const gchar *property_name, GType type);
 	void (*stash_group_display)(struct StashGroup *group, GtkWidget *owner);
 	void (*stash_group_update)(struct StashGroup *group, GtkWidget *owner);
+	void (*stash_group_free_settings)(struct StashGroup *group);
 }
 StashFuncs;
 
@@ -697,6 +721,19 @@ typedef struct SymbolsFuncs
 }
 SymbolsFuncs;
 
+/* See build.h */
+typedef struct BuildFuncs
+{
+	void (*build_activate_menu_item)(const GeanyBuildGroup grp, const guint cmd);
+	const gchar *(*build_get_current_menu_item)(const GeanyBuildGroup grp, const guint cmd,
+			const GeanyBuildCmdEntries field);
+	void (*build_remove_menu_item)(const GeanyBuildSource src, const GeanyBuildGroup grp,
+			const gint cmd);
+	void (*build_set_menu_item)(const GeanyBuildSource src, const GeanyBuildGroup grp,
+			const guint cmd, const GeanyBuildCmdEntries field, const gchar *value);
+	guint (*build_get_group_count)(const GeanyBuildGroup grp);
+}
+BuildFuncs;
 
 /* Deprecated aliases */
 #ifndef GEANY_DISABLE_DEPRECATED
@@ -713,4 +750,6 @@ SymbolsFuncs;
 
 #endif	/* GEANY_DISABLE_DEPRECATED */
 
+G_END_DECLS
+
 #endif
