@@ -1,8 +1,8 @@
 /*
  *      prefs.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,15 +14,16 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef GEANY_PREFS_H
 #define GEANY_PREFS_H 1
 
+G_BEGIN_DECLS
+
 /** General Preferences dialog settings. */
 typedef struct GeanyPrefs
 {
@@ -42,12 +43,13 @@ GeanyPrefs;
 extern GeanyPrefs prefs;
 
 
+/** Tools preferences */
 typedef struct GeanyToolPrefs
 {
-	gchar			*browser_cmd;
-	gchar			*term_cmd;
-	gchar			*grep_cmd;
-	gchar			*context_action_cmd;
+	gchar			*browser_cmd;			/**< web browser command */
+	gchar			*term_cmd;				/**< terminal emulator command */
+	gchar			*grep_cmd;				/**< grep command */
+	gchar			*context_action_cmd;	/**< context action command */
 }
 GeanyToolPrefs;
 
@@ -56,4 +58,8 @@ extern GeanyToolPrefs tool_prefs;
 
 void prefs_show_dialog(void);
 
+void prefs_kb_search_name(const gchar *search);
+
+G_END_DECLS
+
 #endif
