@@ -1,8 +1,8 @@
 /*
  *      sidebar.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: sidebar.c 5003 2010-06-06 17:21:52Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -43,6 +41,7 @@
 #include "stash.h"
 #include "keyfile.h"
 #include "sciwrappers.h"
+#include "search.h"
 
 #include <gdk/gdkkeysyms.h>
 
@@ -57,14 +56,11 @@ static struct
 	GtkWidget *save;
 	GtkWidget *reload;
 	GtkWidget *show_paths;
+	GtkWidget *find_in_files;
+	GtkWidget *expand_all;
+	GtkWidget *collapse_all;
 }
-doc_items = {NULL, NULL, NULL, NULL};
-
-static struct
-{
-	GtkTreeSelection *selection;
-	guint keyval;
-} selection_change = {NULL, 0};
+doc_items = {NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 
 enum
 {
@@ -95,9 +91,7 @@ static gboolean documents_show_paths;
 static GtkWidget *tag_window;	/* scrolled window that holds the symbol list GtkTreeView */
 
 /* callback prototypes */
-static gboolean on_openfiles_tree_selection_changed(gpointer data);
 static void on_openfiles_document_action(GtkMenuItem *menuitem, gpointer user_data);
-static gboolean on_taglist_tree_selection_changed(gpointer data);
 static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event,
 		gpointer user_data);
 static gboolean sidebar_key_press_cb(GtkWidget *widget, GdkEventKey *event,
@@ -142,16 +136,11 @@ static void prepare_taglist(GtkWidget *tree, GtkTreeStore *store)
 	g_signal_connect(tree, "key-press-event",
 		G_CALLBACK(sidebar_key_press_cb), NULL);
 
-	if (gtk_check_version(2, 12, 0) == NULL)
-	{
-		g_object_set(tree, "show-expanders", interface_prefs.show_symbol_list_expanders, NULL);
-		if (! interface_prefs.show_symbol_list_expanders)
-			g_object_set(tree, "level-indentation", 10, NULL);
-		/* Tooltips */
-		g_object_set(tree,
-			"has-tooltip", TRUE,
-			"tooltip-column", SYMBOLS_COLUMN_TOOLTIP, NULL);
-	}
+	gtk_tree_view_set_show_expanders(GTK_TREE_VIEW(tree), interface_prefs.show_symbol_list_expanders);
+	if (! interface_prefs.show_symbol_list_expanders)
+		gtk_tree_view_set_level_indentation(GTK_TREE_VIEW(tree), 10);
+	/* Tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(tree), SYMBOLS_COLUMN_TOOLTIP);
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
@@ -196,8 +185,23 @@ static void create_default_tag_tree(void)
 /* update = rescan the tags for doc->filename */
 void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 {
-	if (gtk_bin_get_child(GTK_BIN(tag_window)))
-		gtk_container_remove(GTK_CONTAINER(tag_window), gtk_bin_get_child(GTK_BIN(tag_window)));
+	GtkWidget *child = gtk_bin_get_child(GTK_BIN(tag_window));
+
+	g_return_if_fail(doc == NULL || doc->is_valid);
+
+	/* changes the tree view to the given one, trying not to do useless changes */
+	#define CHANGE_TREE(new_child) \
+		G_STMT_START { \
+			/* only change the tag tree if it's actually not the same (to avoid flickering) and if
+			 * it's the one of the current document (to avoid problems when e.g. reloading
+			 * configuration files */ \
+			if (child != new_child && doc == document_get_current()) \
+			{ \
+				if (child) \
+					gtk_container_remove(GTK_CONTAINER(tag_window), child); \
+				gtk_container_add(GTK_CONTAINER(tag_window), new_child); \
+			} \
+		} G_STMT_END
 
 	if (tv.default_tag_tree == NULL)
 		create_default_tag_tree();
@@ -205,7 +209,7 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 	/* show default empty tag tree if there are no tags */
 	if (doc == NULL || doc->file_type == NULL || ! filetype_has_tags(doc->file_type))
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), tv.default_tag_tree);
+		CHANGE_TREE(tv.default_tag_tree);
 		return;
 	}
 
@@ -214,7 +218,7 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 		if (doc->priv->tag_tree == NULL)
 		{
 			doc->priv->tag_store = gtk_tree_store_new(
-				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
+				SYMBOLS_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, TM_TYPE_TAG, G_TYPE_STRING);
 			doc->priv->tag_tree = gtk_tree_view_new();
 			prepare_taglist(doc->priv->tag_tree, doc->priv->tag_store);
 			gtk_widget_show(doc->priv->tag_tree);
@@ -226,12 +230,14 @@ void sidebar_update_tag_list(GeanyDocument *doc, gboolean update)
 
 	if (doc->has_tags)
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), doc->priv->tag_tree);
+		CHANGE_TREE(doc->priv->tag_tree);
 	}
 	else
 	{
-		gtk_container_add(GTK_CONTAINER(tag_window), tv.default_tag_tree);
+		CHANGE_TREE(tv.default_tag_tree);
 	}
+
+	#undef CHANGE_TREE
 }
 
 
@@ -270,7 +276,7 @@ static void prepare_openfiles(void)
 
 	/* store the icon and the short filename to show, and the index as reference,
 	 * the colour (black/red/green) and the full name for the tooltip */
-	store_openfiles = gtk_tree_store_new(5, G_TYPE_STRING, G_TYPE_STRING,
+	store_openfiles = gtk_tree_store_new(5, GDK_TYPE_PIXBUF, G_TYPE_STRING,
 		G_TYPE_POINTER, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tv.tree_openfiles), GTK_TREE_MODEL(store_openfiles));
 
@@ -285,7 +291,7 @@ static void prepare_openfiles(void)
 	g_object_set(text_renderer, "ellipsize", PANGO_ELLIPSIZE_MIDDLE, NULL);
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_pack_start(column, icon_renderer, FALSE);
-	gtk_tree_view_column_set_attributes(column, icon_renderer, "stock-id", DOCUMENTS_ICON, NULL);
+	gtk_tree_view_column_set_attributes(column, icon_renderer, "pixbuf", DOCUMENTS_ICON, NULL);
 	gtk_tree_view_column_pack_start(column, text_renderer, TRUE);
 	gtk_tree_view_column_set_attributes(column, text_renderer, "text", DOCUMENTS_SHORTNAME,
 		"foreground-gdk", DOCUMENTS_COLOR, NULL);
@@ -302,9 +308,8 @@ static void prepare_openfiles(void)
 
 	ui_widget_modify_font_from_string(tv.tree_openfiles, interface_prefs.tagbar_font);
 
-	/* GTK 2.12 tooltips */
-	if (gtk_check_version(2, 12, 0) == NULL)
-		g_object_set(tv.tree_openfiles, "has-tooltip", TRUE, "tooltip-column", DOCUMENTS_FILENAME, NULL);
+	/* tooltips */
+	gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(tv.tree_openfiles), DOCUMENTS_FILENAME);
 
 	/* selection handling */
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
@@ -333,49 +338,89 @@ static gboolean find_tree_iter_dir(GtkTreeIter *iter, const gchar *dir)
 
 	gtk_tree_model_get(GTK_TREE_MODEL(store_openfiles), iter, DOCUMENTS_SHORTNAME, &name, -1);
 
-	result = utils_str_equal(name, dir);
+	result = utils_filenamecmp(name, dir) == 0;
 	g_free(name);
 
 	return result;
 }
 
 
-static GtkTreeIter *get_doc_parent(GeanyDocument *doc)
+static gboolean utils_filename_has_prefix(const gchar *str, const gchar *prefix)
 {
-	gchar *tmp_dirname;
+	gchar *head = g_strndup(str, strlen(prefix));
+	gboolean ret = utils_filenamecmp(head, prefix) == 0;
+
+	g_free(head);
+	return ret;
+}
+
+
+static gchar *get_doc_folder(const gchar *path)
+{
+	gchar *tmp_dirname = g_strdup(path);
 	gchar *project_base_path;
 	gchar *dirname = NULL;
-	static GtkTreeIter parent;
-	GtkTreeModel *model = GTK_TREE_MODEL(store_openfiles);
+	const gchar *home_dir = g_get_home_dir();
+	const gchar *rest;
 
-	if (!documents_show_paths)
-		return NULL;
-
-	tmp_dirname = g_path_get_dirname(DOC_FILENAME(doc));
 	/* replace the project base path with the project name */
 	project_base_path = project_get_base_path();
+
 	if (project_base_path != NULL)
 	{
 		gsize len = strlen(project_base_path);
-		const gchar *rest;
 
+		/* remove trailing separator so we can match base path exactly */
 		if (project_base_path[len-1] == G_DIR_SEPARATOR)
-			project_base_path[len-1] = '\0';
+			project_base_path[--len] = '\0';
 
 		/* check whether the dir name matches or uses the project base path */
-		if (g_str_has_prefix(tmp_dirname, project_base_path))
+		if (utils_filename_has_prefix(tmp_dirname, project_base_path))
 		{
 			rest = tmp_dirname + len;
 			if (*rest == G_DIR_SEPARATOR || *rest == '\0')
+			{
 				dirname = g_strdup_printf("%s%s", app->project->name, rest);
+			}
 		}
 		g_free(project_base_path);
 	}
 	if (dirname == NULL)
+	{
 		dirname = tmp_dirname;
+
+		/* If matches home dir, replace with tilde */
+		if (!EMPTY(home_dir) && utils_filename_has_prefix(dirname, home_dir))
+		{
+			rest = dirname + strlen(home_dir);
+			if (*rest == G_DIR_SEPARATOR || *rest == '\0')
+			{
+				dirname = g_strdup_printf("~%s", rest);
+				g_free(tmp_dirname);
+			}
+		}
+	}
 	else
 		g_free(tmp_dirname);
 
+	return dirname;
+}
+
+
+static GtkTreeIter *get_doc_parent(GeanyDocument *doc)
+{
+	gchar *path;
+	gchar *dirname = NULL;
+	static GtkTreeIter parent;
+	GtkTreeModel *model = GTK_TREE_MODEL(store_openfiles);
+	static GdkPixbuf *dir_icon = NULL;
+
+	if (!documents_show_paths)
+		return NULL;
+
+	path = g_path_get_dirname(DOC_FILENAME(doc));
+	dirname = get_doc_folder(path);
+
 	if (gtk_tree_model_get_iter_first(model, &parent))
 	{
 		do
@@ -383,17 +428,23 @@ static GtkTreeIter *get_doc_parent(GeanyDocument *doc)
 			if (find_tree_iter_dir(&parent, dirname))
 			{
 				g_free(dirname);
+				g_free(path);
 				return &parent;
 			}
 		}
 		while (gtk_tree_model_iter_next(model, &parent));
 	}
 	/* no match, add dir parent */
+	if (!dir_icon)
+		dir_icon = ui_get_mime_icon("inode/directory", GTK_ICON_SIZE_MENU);
+
 	gtk_tree_store_append(store_openfiles, &parent, NULL);
-	gtk_tree_store_set(store_openfiles, &parent, DOCUMENTS_ICON, GTK_STOCK_DIRECTORY,
+	gtk_tree_store_set(store_openfiles, &parent, DOCUMENTS_ICON, dir_icon,
+		DOCUMENTS_FILENAME, path,
 		DOCUMENTS_SHORTNAME, doc->file_name ? dirname : GEANY_STRING_UNTITLED, -1);
 
 	g_free(dirname);
+	g_free(path);
 	return &parent;
 }
 
@@ -406,6 +457,7 @@ void sidebar_openfiles_add(GeanyDocument *doc)
 	GtkTreeIter *parent = get_doc_parent(doc);
 	gchar *basename;
 	const GdkColor *color = document_get_status_color(doc);
+	static GdkPixbuf *file_icon = NULL;
 
 	gtk_tree_store_append(store_openfiles, iter, parent);
 
@@ -419,8 +471,12 @@ void sidebar_openfiles_add(GeanyDocument *doc)
 		gtk_tree_view_expand_row(GTK_TREE_VIEW(tv.tree_openfiles), path, TRUE);
 		gtk_tree_path_free(path);
 	}
+	if (!file_icon)
+		file_icon = ui_get_mime_icon("text/plain", GTK_ICON_SIZE_MENU);
+
 	basename = g_path_get_basename(DOC_FILENAME(doc));
-	gtk_tree_store_set(store_openfiles, iter, DOCUMENTS_ICON, GTK_STOCK_FILE,
+	gtk_tree_store_set(store_openfiles, iter,
+		DOCUMENTS_ICON, (doc->file_type && doc->file_type->icon) ? doc->file_type->icon : file_icon,
 		DOCUMENTS_SHORTNAME, basename, DOCUMENTS_DOCUMENT, doc, DOCUMENTS_COLOR, color,
 		DOCUMENTS_FILENAME, DOC_FILENAME(doc), -1);
 	g_free(basename);
@@ -449,10 +505,13 @@ void sidebar_openfiles_update(GeanyDocument *doc)
 
 	if (utils_str_equal(fname, DOC_FILENAME(doc)))
 	{
-		/* just update color */
+		/* just update color and the icon */
 		const GdkColor *color = document_get_status_color(doc);
+		GdkPixbuf *icon = doc->file_type->icon;
 
 		gtk_tree_store_set(store_openfiles, iter, DOCUMENTS_COLOR, color, -1);
+		if (icon)
+			gtk_tree_store_set(store_openfiles, iter, DOCUMENTS_ICON, icon, -1);
 	}
 	else
 	{
@@ -472,20 +531,14 @@ void sidebar_openfiles_update(GeanyDocument *doc)
 }
 
 
-void sidebar_openfiles_update_all()
+void sidebar_openfiles_update_all(void)
 {
-	guint i, page_count;
-	GeanyDocument *doc;
+	guint i;
 
 	gtk_tree_store_clear(store_openfiles);
-	page_count = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
-	for (i = 0; i < page_count; i++)
+	foreach_document (i)
 	{
-		doc = document_get_from_page(i);
-		if (G_UNLIKELY(doc == NULL))
-			continue;
-
-		sidebar_openfiles_add(doc);
+		sidebar_openfiles_add(documents[i]);
 	}
 }
 
@@ -496,12 +549,9 @@ void sidebar_remove_document(GeanyDocument *doc)
 
 	if (GTK_IS_WIDGET(doc->priv->tag_tree))
 	{
-		gtk_widget_destroy(doc->priv->tag_tree);
-		if (GTK_IS_TREE_VIEW(doc->priv->tag_tree))
-		{
-			/* Because it was ref'd in sidebar_update_tag_list, it needs unref'ing */
-			g_object_unref((gpointer)doc->priv->tag_tree);
-		}
+		gtk_widget_destroy(doc->priv->tag_tree); /* make GTK release its references, if any */
+		/* Because it was ref'd in sidebar_update_tag_list, it needs unref'ing */
+		g_object_unref(doc->priv->tag_tree);
 		doc->priv->tag_tree = NULL;
 	}
 }
@@ -540,23 +590,31 @@ void sidebar_add_common_menu_items(GtkMenu *menu)
 
 	item = gtk_check_menu_item_new_with_mnemonic(_("Show S_ymbol List"));
 	gtk_container_add(GTK_CONTAINER(menu), item);
+#if GTK_CHECK_VERSION(3, 0, 0)
+	g_signal_connect(item, "draw", G_CALLBACK(on_sidebar_display_symbol_list_show), NULL);
+#else
 	g_signal_connect(item, "expose-event",
 			G_CALLBACK(on_sidebar_display_symbol_list_show), NULL);
+#endif
 	gtk_widget_show(item);
 	g_signal_connect(item, "activate",
 			G_CALLBACK(on_list_symbol_activate), NULL);
 
 	item = gtk_check_menu_item_new_with_mnemonic(_("Show _Document List"));
 	gtk_container_add(GTK_CONTAINER(menu), item);
+#if GTK_CHECK_VERSION(3, 0, 0)
+	g_signal_connect(item, "draw", G_CALLBACK(on_sidebar_display_open_files_show), NULL);
+#else
 	g_signal_connect(item, "expose-event",
 			G_CALLBACK(on_sidebar_display_open_files_show), NULL);
+#endif
 	gtk_widget_show(item);
 	g_signal_connect(item, "activate",
 			G_CALLBACK(on_list_document_activate), NULL);
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("H_ide Sidebar"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		gtk_image_new_from_stock("gtk-close", GTK_ICON_SIZE_MENU));
+		gtk_image_new_from_stock(GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_hide_sidebar), NULL);
@@ -586,13 +644,49 @@ static void on_list_symbol_activate(GtkCheckMenuItem *item, gpointer user_data)
 }
 
 
+static void on_find_in_files(GtkMenuItem *menuitem, gpointer user_data)
+{
+	GtkTreeSelection *treesel;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	GeanyDocument *doc;
+	gchar *dir;
+
+	treesel = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv.tree_openfiles));
+	if (!gtk_tree_selection_get_selected(treesel, &model, &iter))
+		return;
+	gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc, -1);
+
+	if (!doc)
+	{
+		gtk_tree_model_get(model, &iter, DOCUMENTS_FILENAME, &dir, -1);
+	}
+	else
+		dir = g_path_get_dirname(DOC_FILENAME(doc));
+
+	search_show_find_in_files_dialog(dir);
+	g_free(dir);
+}
+
+
+static void on_openfiles_expand_collapse(GtkMenuItem *menuitem, gpointer user_data)
+{
+	gboolean expand = GPOINTER_TO_INT(user_data);
+
+	if (expand)
+		gtk_tree_view_expand_all(GTK_TREE_VIEW(tv.tree_openfiles));
+	else
+		gtk_tree_view_collapse_all(GTK_TREE_VIEW(tv.tree_openfiles));
+}
+
+
 static void create_openfiles_popup_menu(void)
 {
 	GtkWidget *item;
 
 	openfiles_popup_menu = gtk_menu_new();
 
-	item = gtk_image_menu_item_new_from_stock("gtk-close", NULL);
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLOSE, NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -603,7 +697,7 @@ static void create_openfiles_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 
-	item = gtk_image_menu_item_new_from_stock("gtk-save", NULL);
+	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, NULL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -612,7 +706,7 @@ static void create_openfiles_popup_menu(void)
 
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Reload"));
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		gtk_image_new_from_stock("gtk-revert-to-saved", GTK_ICON_SIZE_MENU));
+		gtk_image_new_from_stock(GTK_STOCK_REVERT_TO_SAVED, GTK_ICON_SIZE_MENU));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 	g_signal_connect(item, "activate",
@@ -623,12 +717,39 @@ static void create_openfiles_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
 
+	item = ui_image_menu_item_new(GTK_STOCK_FIND, _("_Find in Files..."));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(on_find_in_files), NULL);
+	doc_items.find_in_files = item;
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
+
 	doc_items.show_paths = gtk_check_menu_item_new_with_mnemonic(_("Show _Paths"));
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths), documents_show_paths);
 	gtk_widget_show(doc_items.show_paths);
 	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.show_paths);
 	g_signal_connect(doc_items.show_paths, "activate",
 			G_CALLBACK(on_openfiles_show_paths_activate), NULL);
 
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), item);
+
+	doc_items.expand_all = ui_image_menu_item_new(GTK_STOCK_ADD, _("_Expand All"));
+	gtk_widget_show(doc_items.expand_all);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.expand_all);
+	g_signal_connect(doc_items.expand_all, "activate",
+					 G_CALLBACK(on_openfiles_expand_collapse), GINT_TO_POINTER(TRUE));
+
+	doc_items.collapse_all = ui_image_menu_item_new(GTK_STOCK_REMOVE, _("_Collapse All"));
+	gtk_widget_show(doc_items.collapse_all);
+	gtk_container_add(GTK_CONTAINER(openfiles_popup_menu), doc_items.collapse_all);
+	g_signal_connect(doc_items.collapse_all, "activate",
+					 G_CALLBACK(on_openfiles_expand_collapse), GINT_TO_POINTER(FALSE));
+
 	sidebar_add_common_menu_items(GTK_MENU(openfiles_popup_menu));
 }
 
@@ -744,39 +865,38 @@ static void change_focus_to_editor(GeanyDocument *doc, GtkWidget *source_widget)
 }
 
 
-static gboolean on_openfiles_tree_selection_changed(gpointer data)
+static gboolean openfiles_go_to_selection(GtkTreeSelection *selection, guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GeanyDocument *doc = NULL;
 
 	/* use switch_notebook_page to ignore changing the notebook page because it is already done */
-	if (gtk_tree_selection_get_selected(selection_change.selection, &model, &iter) && ! ignore_callback)
+	if (gtk_tree_selection_get_selected(selection, &model, &iter) && ! ignore_callback)
 	{
 		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc, -1);
 		if (! doc)
 			return FALSE;	/* parent */
 
 		/* switch to the doc and grab the focus */
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.notebook),
-			gtk_notebook_page_num(GTK_NOTEBOOK(main_widgets.notebook),
-			(GtkWidget*) doc->editor->sci));
-		if (selection_change.keyval != GDK_space)
+		document_show_tab(doc);
+		if (keyval != GDK_space)
 			change_focus_to_editor(doc, tv.tree_openfiles);
 	}
 	return FALSE;
 }
 
 
-static gboolean on_taglist_tree_selection_changed(gpointer data)
+static gboolean taglist_go_to_selection(GtkTreeSelection *selection, guint keyval, guint state)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gint line = 0;
+	gboolean handled = TRUE;
 
-	if (gtk_tree_selection_get_selected(selection_change.selection, &model, &iter))
+	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
-		const TMTag *tag;
+		TMTag *tag;
 
 		gtk_tree_model_get(model, &iter, SYMBOLS_COLUMN_TAG, &tag, -1);
 		if (! tag)
@@ -790,19 +910,15 @@ static gboolean on_taglist_tree_selection_changed(gpointer data)
 			if (doc != NULL)
 			{
 				navqueue_goto_line(doc, doc, line);
-				if (selection_change.keyval != GDK_space)
+				if (keyval != GDK_space && ! (state & GDK_CONTROL_MASK))
 					change_focus_to_editor(doc, NULL);
+				else
+					handled = FALSE;
 			}
 		}
+		tm_tag_unref(tag);
 	}
-	return FALSE;
-}
-
-
-static void update_selection_change(GtkTreeSelection *selection, guint keyval)
-{
-	selection_change.selection = selection;
-	selection_change.keyval = keyval;
+	return handled;
 }
 
 
@@ -812,16 +928,22 @@ static gboolean sidebar_key_press_cb(GtkWidget *widget, GdkEventKey *event,
 	may_steal_focus = FALSE;
 	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
 	{
+		GtkWidgetClass *widget_class = GTK_WIDGET_GET_CLASS(widget);
 		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 		may_steal_focus = TRUE;
-		/* delay the query of selection state because this callback is executed before GTK
-		 * changes the selection (g_signal_connect_after would be better but it doesn't work) */
-		update_selection_change(selection, event->keyval);
 
-		if (widget ==  tv.tree_openfiles) /* tag and doc list have separate handlers */
-			g_idle_add(on_openfiles_tree_selection_changed, NULL);
+		/* force the TreeView handler to run before us for it to do its job (selection & stuff).
+		 * doing so will prevent further handlers to be run in most cases, but the only one is our
+		 * own, so guess it's fine. */
+		if (widget_class->key_press_event)
+			widget_class->key_press_event(widget, event);
+
+		if (widget == tv.tree_openfiles) /* tag and doc list have separate handlers */
+			openfiles_go_to_selection(selection, event->keyval);
 		else
-			g_idle_add(on_taglist_tree_selection_changed, NULL);
+			taglist_go_to_selection(selection, event->keyval, event->state);
+
+		return TRUE;
 	}
 	return FALSE;
 }
@@ -831,6 +953,14 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 		G_GNUC_UNUSED gpointer user_data)
 {
 	GtkTreeSelection *selection;
+	GtkWidgetClass *widget_class = GTK_WIDGET_GET_CLASS(widget);
+	gboolean handled = FALSE;
+
+	/* force the TreeView handler to run before us for it to do its job (selection & stuff).
+	 * doing so will prevent further handlers to be run in most cases, but the only one is our own,
+	 * so guess it's fine. */
+	if (widget_class->button_press_event)
+		handled = widget_class->button_press_event(widget, event);
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
 	may_steal_focus = TRUE;
@@ -858,14 +988,13 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 	}
 	else if (event->button == 1)
 	{	/* allow reclicking of taglist treeview item */
-		/* delay the query of selection state because this callback is executed before GTK
-		 * changes the selection (g_signal_connect_after would be better but it doesn't work) */
-		update_selection_change(selection, 0);
-
 		if (widget == tv.tree_openfiles)
-			g_idle_add(on_openfiles_tree_selection_changed, NULL);
+		{
+			openfiles_go_to_selection(selection, 0);
+			handled = TRUE;
+		}
 		else
-			g_idle_add(on_taglist_tree_selection_changed, NULL);
+			handled = taglist_go_to_selection(selection, 0, event->state);
 	}
 	else if (event->button == 3)
 	{
@@ -884,8 +1013,9 @@ static gboolean sidebar_button_press_cb(GtkWidget *widget, GdkEventButton *event
 			gtk_menu_popup(GTK_MENU(tv.popup_taglist), NULL, NULL, NULL, NULL,
 					event->button, event->time);
 		}
+		handled = TRUE;
 	}
-	return FALSE;
+	return handled;
 }
 
 
@@ -904,7 +1034,7 @@ static void documents_menu_update(GtkTreeSelection *selection)
 		gtk_tree_model_get(model, &iter, DOCUMENTS_DOCUMENT, &doc,
 			DOCUMENTS_SHORTNAME, &shortname, -1);
 	}
-	path = NZV(shortname) &&
+	path = !EMPTY(shortname) &&
 		(g_path_is_absolute(shortname) ||
 		(app->project && g_str_has_prefix(shortname, app->project->name)));
 
@@ -912,14 +1042,16 @@ static void documents_menu_update(GtkTreeSelection *selection)
 	gtk_widget_set_sensitive(doc_items.close, sel);
 	gtk_widget_set_sensitive(doc_items.save, (doc && doc->real_path) || path);
 	gtk_widget_set_sensitive(doc_items.reload, doc && doc->real_path);
+	gtk_widget_set_sensitive(doc_items.find_in_files, sel);
 	g_free(shortname);
 
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths),
-		documents_show_paths);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(doc_items.show_paths), documents_show_paths);
+	gtk_widget_set_sensitive(doc_items.expand_all, documents_show_paths);
+	gtk_widget_set_sensitive(doc_items.collapse_all, documents_show_paths);
 }
 
 
-StashGroup *stash_group = NULL;
+static StashGroup *stash_group = NULL;
 
 static void on_load_settings(void)
 {
@@ -954,15 +1086,14 @@ void sidebar_init(void)
 	g_signal_connect(geany_object, "load-settings", on_load_settings, NULL);
 	g_signal_connect(geany_object, "save-settings", on_save_settings, NULL);
 
+	g_signal_connect(main_widgets.sidebar_notebook, "page-added",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
+	g_signal_connect(main_widgets.sidebar_notebook, "page-removed",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
+	/* tabs may have changed when sidebar is reshown */
+	g_signal_connect(main_widgets.sidebar_notebook, "show",
+		G_CALLBACK(sidebar_tabs_show_hide), NULL);
 
-	if (gtk_check_version(2, 10, 0) == NULL)
-	{
-		g_signal_connect(main_widgets.sidebar_notebook, "page-added",
-			G_CALLBACK(sidebar_tabs_show_hide), NULL);
-
-		g_signal_connect(main_widgets.sidebar_notebook, "page-removed",
-			G_CALLBACK(sidebar_tabs_show_hide), NULL);
-	}
 	sidebar_tabs_show_hide(GTK_NOTEBOOK(main_widgets.sidebar_notebook), NULL, 0, NULL);
 }
 
@@ -972,11 +1103,8 @@ void sidebar_finalize(void)
 {
 	if (WIDGET(tv.default_tag_tree))
 	{
-		g_object_unref(tv.default_tag_tree);
-		/* This is not exactly clean, default_tag_tree's ref_count is 2 when it is shown,
-		 * 1 oherwise. We should probably handle the ref_count more accurate. */
-		if (WIDGET(tv.default_tag_tree))
-			gtk_widget_destroy(tv.default_tag_tree);
+		gtk_widget_destroy(tv.default_tag_tree); /* make GTK release its references, if any... */
+		g_object_unref(tv.default_tag_tree); /* ...and release our own */
 	}
 	if (WIDGET(tv.popup_taglist))
 		gtk_widget_destroy(tv.popup_taglist);
@@ -1013,15 +1141,12 @@ void sidebar_focus_symbols_tab(void)
 static void sidebar_tabs_show_hide(GtkNotebook *notebook, GtkWidget *child,
 								   guint page_num, gpointer data)
 {
-	if (gtk_check_version(2, 10, 0) == NULL)
-	{
-		gint tabs = gtk_notebook_get_n_pages(notebook);
+	gint tabs = gtk_notebook_get_n_pages(notebook);
 
-		if (interface_prefs.sidebar_symbol_visible == FALSE)
-			tabs--;
-		if (interface_prefs.sidebar_openfiles_visible == FALSE)
-			tabs--;
+	if (interface_prefs.sidebar_symbol_visible == FALSE)
+		tabs--;
+	if (interface_prefs.sidebar_openfiles_visible == FALSE)
+		tabs--;
 
-		gtk_notebook_set_show_tabs(notebook, (tabs > 1));
-	}
+	gtk_notebook_set_show_tabs(notebook, (tabs > 1));
 }
