@@ -1,8 +1,8 @@
 /*
  *      templates.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,15 +14,13 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: templates.c 5390 2010-11-10 13:13:28Z ntrel $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
- * Templates to insert into the current document, or filetype templates to create a new
+ * Templates to insert into the current document, or file templates to create a new
  * document from.
  */
 
@@ -35,6 +33,7 @@
 #include "support.h"
 #include "utils.h"
 #include "document.h"
+#include "encodings.h"
 #include "editor.h"
 #include "filetypes.h"
 #include "ui_utils.h"
@@ -50,10 +49,6 @@ static GtkWidget *new_with_template_menu = NULL;	/* submenu used for both file m
 /* TODO: implement custom insertion templates instead? */
 static gchar *templates[GEANY_MAX_TEMPLATES];
 
-/* We should probably remove filetype templates support soon - users can use custom
- * file templates instead. */
-static gchar *ft_templates[GEANY_MAX_BUILT_IN_FILETYPES] = {NULL};
-
 
 static void replace_static_values(GString *text);
 static gchar *get_template_fileheader(GeanyFiletype *ft);
@@ -64,20 +59,25 @@ static void templates_replace_command(GString *text, const gchar *file_name,
 	const gchar *file_type, const gchar *func_name);
 
 
-/* some simple macros to reduce code size and make the code readable */
-#define TEMPLATES_GET_FILENAME(shortname) \
-	g_strconcat(app->configdir, \
-		G_DIR_SEPARATOR_S GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S, shortname, NULL)
-
-
 static gchar *read_file(const gchar *locale_fname)
 {
 	gchar *contents;
+	gsize length;
 	GString *str;
 
-	if (!g_file_get_contents(locale_fname, &contents, NULL, NULL))
+	if (! g_file_get_contents(locale_fname, &contents, &length, NULL))
 		return NULL;
 
+	if (! encodings_convert_to_utf8_auto(&contents, &length, NULL, NULL, NULL, NULL))
+	{
+		gchar *utf8_fname = utils_get_utf8_from_locale(locale_fname);
+
+		ui_set_statusbar(TRUE, _("Failed to convert template file \"%s\" to UTF-8"), utf8_fname);
+		g_free(utf8_fname);
+		g_free(contents);
+		return NULL;
+	}
+
 	str = g_string_new(contents);
 	g_free(contents);
 
@@ -89,44 +89,26 @@ static gchar *read_file(const gchar *locale_fname)
 
 static void read_template(const gchar *name, gint id)
 {
-	gchar *fname = TEMPLATES_GET_FILENAME(name);
+	gchar *fname = g_build_path(G_DIR_SEPARATOR_S, app->configdir,
+		GEANY_TEMPLATES_SUBDIR, name, NULL);
 
 	/* try system if user template doesn't exist */
 	if (!g_file_test(fname, G_FILE_TEST_EXISTS))
-		setptr(fname, g_strconcat(app->datadir,
-			G_DIR_SEPARATOR_S GEANY_TEMPLATES_SUBDIR G_DIR_SEPARATOR_S, name, NULL));
+		SETPTR(fname, g_build_path(G_DIR_SEPARATOR_S, app->datadir,
+			GEANY_TEMPLATES_SUBDIR, name, NULL));
 
 	templates[id] = read_file(fname);
 	g_free(fname);
 }
 
 
-/* FIXME the callers should use GStrings instead of char arrays */
-static gchar *replace_all(gchar *text, const gchar *year, const gchar *date, const gchar *datetime)
-{
-	GString *str;
-
-	if (text == NULL)
-		return NULL;
-
-	str = g_string_new(text);
-
-	g_free(text);
-	templates_replace_valist(str,
-		"{year}", year,
-		"{date}", date,
-		"{datetime}", datetime,
-		NULL);
-
-	return g_string_free(str, FALSE);
-}
-
-
 /* called when inserting templates into an existing document */
 static void convert_eol_characters(GString *template, GeanyDocument *doc)
 {
 	gint doc_eol_mode;
 
+	g_return_if_fail(doc == NULL || doc->is_valid);
+
 	if (doc == NULL)
 		doc = document_get_current();
 
@@ -137,89 +119,18 @@ static void convert_eol_characters(GString *template, GeanyDocument *doc)
 }
 
 
-static void init_general_templates(const gchar *year, const gchar *date, const gchar *datetime)
+static void init_general_templates(void)
 {
-	guint id;
-
 	/* read the contents */
 	read_template("fileheader", GEANY_TEMPLATE_FILEHEADER);
 	read_template("gpl", GEANY_TEMPLATE_GPL);
 	read_template("bsd", GEANY_TEMPLATE_BSD);
 	read_template("function", GEANY_TEMPLATE_FUNCTION);
 	read_template("changelog", GEANY_TEMPLATE_CHANGELOG);
-
-	/* FIXME: we should replace the dates on insertion, not on loading */
-	for (id = 0; id < GEANY_MAX_TEMPLATES; id++)
-		templates[id] = replace_all(templates[id], year, date, datetime);
 }
 
 
-static void init_ft_templates(const gchar *year, const gchar *date, const gchar *datetime)
-{
-	filetype_id ft_id;
-
-	for (ft_id = 0; ft_id < GEANY_MAX_BUILT_IN_FILETYPES; ft_id++)
-	{
-		gchar *ext = (ft_id != GEANY_FILETYPES_NONE) ?
-			filetypes_get_conf_extension(ft_id) : g_strdup("none");
-		gchar *shortname = g_strconcat("filetype.", ext, NULL);
-		gchar *fname = TEMPLATES_GET_FILENAME(shortname);
-
-		ft_templates[ft_id] = read_file(fname);
-		ft_templates[ft_id] = replace_all(ft_templates[ft_id], year, date, datetime);
-
-		g_free(fname);
-		g_free(shortname);
-		g_free(ext);
-	}
-}
-
-
-static void
-on_new_with_filetype_template(GtkMenuItem *menuitem, gpointer user_data)
-{
-	GeanyFiletype *ft = user_data;
-	gchar *template = templates_get_template_new_file(ft);
-
-	document_new_file(NULL, ft, template);
-	g_free(template);
-}
-
-
-/* TODO: remove filetype template support after 0.19 */
-static gboolean create_new_filetype_items(void)
-{
-	GSList *node;
-	gboolean ret = FALSE;
-	GtkWidget *menu = NULL;
-
-	foreach_slist(node, filetypes_by_title)
-	{
-		GeanyFiletype *ft = node->data;
-		GtkWidget *item;
-
-		if (ft->id >= GEANY_MAX_BUILT_IN_FILETYPES || ft_templates[ft->id] == NULL)
-			continue;
-
-		if (!menu)
-		{
-			item = gtk_menu_item_new_with_label(_("Old"));
-			menu = gtk_menu_new();
-			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
-			gtk_widget_show_all(item);
-			gtk_container_add(GTK_CONTAINER(new_with_template_menu), item);
-		}
-		item = gtk_menu_item_new_with_label(ft->title);
-		gtk_widget_show(item);
-		gtk_container_add(GTK_CONTAINER(menu), item);
-		g_signal_connect(item, "activate", G_CALLBACK(on_new_with_filetype_template), ft);
-		ret = TRUE;
-	}
-	return ret;
-}
-
-
-void templates_replace_common(GString *template, const gchar *fname,
+void templates_replace_common(GString *tmpl, const gchar *fname,
 							  GeanyFiletype *ft, const gchar *func_name)
 {
 	gchar *shortname;
@@ -234,18 +145,18 @@ void templates_replace_common(GString *template, const gchar *fname,
 	else
 		shortname = g_path_get_basename(fname);
 
-	templates_replace_valist(template,
+	templates_replace_valist(tmpl,
 		"{filename}", shortname,
 		"{project}", app->project ? app->project->name : "",
 		"{description}", app->project ? app->project->description : "",
 		NULL);
 	g_free(shortname);
 
-	templates_replace_default_dates(template);
-	templates_replace_command(template, fname, ft->name, func_name);
+	templates_replace_default_dates(tmpl);
+	templates_replace_command(tmpl, fname, ft->name, func_name);
 	/* Bug: command results could have {ob} {cb} strings in! */
 	/* replace braces last */
-	templates_replace_valist(template,
+	templates_replace_valist(tmpl,
 		"{ob}", "{",
 		"{cb}", "}",
 		NULL);
@@ -290,7 +201,7 @@ on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_dat
 	gchar *path;
 
 	ft = filetypes_detect_from_extension(fname);
-	setptr(fname, utils_get_locale_from_utf8(fname));
+	SETPTR(fname, utils_get_locale_from_utf8(fname));
 
 	/* fname is just the basename from the menu item, so prepend the custom files path */
 	path = g_build_path(G_DIR_SEPARATOR_S, app->configdir, GEANY_TEMPLATES_SUBDIR,
@@ -311,7 +222,7 @@ on_new_with_file_template(GtkMenuItem *menuitem, G_GNUC_UNUSED gpointer user_dat
 	}
 	else
 	{
-		setptr(fname, utils_get_utf8_from_locale(fname));
+		SETPTR(fname, utils_get_utf8_from_locale(fname));
 		ui_set_statusbar(TRUE, _("Could not find file '%s'."), fname);
 	}
 	g_free(template);
@@ -359,19 +270,9 @@ static gboolean add_custom_template_items(void)
 
 static void create_file_template_menu(void)
 {
-	GtkWidget *sep = NULL;
-
 	new_with_template_menu = gtk_menu_new();
+	add_custom_template_items();
 
-	if (add_custom_template_items())
-	{
-		sep = gtk_separator_menu_item_new();
-		gtk_container_add(GTK_CONTAINER(new_with_template_menu), sep);
-	}
-	if (create_new_filetype_items() && sep)
-	{
-		gtk_widget_show(sep);
-	}
 	/* unless the file menu is showing, menu should be in the toolbar widget */
 	geany_menu_button_action_set_menu(GEANY_MENU_BUTTON_ACTION(
 		toolbar_get_action_by_name("New")), new_with_template_menu);
@@ -399,9 +300,9 @@ static void on_file_menu_hide(GtkWidget *item)
 /* reload templates if any file in the templates path is saved */
 static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 {
-	const gchar *path = utils_build_path(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
+	gchar *path = g_build_filename(app->configdir, GEANY_TEMPLATES_SUBDIR, NULL);
 
-	g_return_if_fail(NZV(doc->real_path));
+	g_return_if_fail(!EMPTY(doc->real_path));
 
 	if (strncmp(doc->real_path, path, strlen(path)) == 0)
 	{
@@ -409,23 +310,16 @@ static void on_document_save(G_GNUC_UNUSED GObject *object, GeanyDocument *doc)
 		templates_free_templates();
 		templates_init();
 	}
+	g_free(path);
 }
 
 
 /* warning: also called when reloading template settings */
 void templates_init(void)
 {
-	gchar *year = utils_get_date_time(template_prefs.year_format, NULL);
-	gchar *date = utils_get_date_time(template_prefs.date_format, NULL);
-	gchar *datetime = utils_get_date_time(template_prefs.datetime_format, NULL);
 	static gboolean init_done = FALSE;
 
-	init_general_templates(year, date, datetime);
-	init_ft_templates(year, date, datetime);
-
-	g_free(date);
-	g_free(datetime);
-	g_free(year);
+	init_general_templates();
 
 	create_file_template_menu();
 	/* we hold our own ref for the menu as it has no parent whilst being moved */
@@ -459,10 +353,12 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 	gchar *tmp;
 	gchar *prefix;
 	gchar **lines;
-	guint i, len;
+	gsize i, len;
 	gint template_eol_mode;
 	const gchar *template_eol_char;
 	GeanyFiletype *ft = filetypes_index(filetype_idx);
+	const gchar *co;
+	const gchar *cc;
 
 	g_return_if_fail(comment_text != NULL);
 	g_return_if_fail(ft != NULL);
@@ -470,19 +366,20 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 	template_eol_mode = utils_get_line_endings(comment_text->str, comment_text->len);
 	template_eol_char = utils_get_eol_char(template_eol_mode);
 
-	if (NZV(ft->comment_open))
+	filetype_get_comment_open_close(ft, FALSE, &co, &cc);
+	if (!EMPTY(co))
 	{
-		if (NZV(ft->comment_close))
+		if (!EMPTY(cc))
 		{
-			frame_start = g_strconcat(ft->comment_open, template_eol_char, NULL);
-			frame_end = g_strconcat(ft->comment_close, template_eol_char, NULL);
+			frame_start = g_strconcat(co, template_eol_char, NULL);
+			frame_end = g_strconcat(cc, template_eol_char, NULL);
 			line_prefix = "";
 		}
 		else
 		{
 			frame_start = NULL;
 			frame_end = NULL;
-			line_prefix = ft->comment_open;
+			line_prefix = co;
 		}
 	}
 	else
@@ -493,10 +390,10 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 	}
 
 	/* do some magic to nicely format C-like multi-line comments */
-	if (NZV(frame_start) && frame_start[1] == '*')
+	if (!EMPTY(frame_start) && frame_start[1] == '*')
 	{
 		/* prefix the string with a space */
-		setptr(frame_end, g_strconcat(" ", frame_end, NULL));
+		SETPTR(frame_end, g_strconcat(" ", frame_end, NULL));
 		line_prefix = " *";
 	}
 
@@ -508,12 +405,15 @@ static void make_comment_block(GString *comment_text, gint filetype_idx, guint i
 
 	/* add line_prefix to every line of comment_text */
 	lines = g_strsplit(comment_text->str, template_eol_char, -1);
-	len = g_strv_length(lines) - 1;
-	for (i = 0; i < len; i++)
+	len = g_strv_length(lines);
+	if (len > 0)	/* prevent unsigned wraparound if comment_text is empty */
 	{
-		tmp = lines[i];
-		lines[i] = g_strconcat(prefix, tmp, NULL);
-		g_free(tmp);
+		for (i = 0; i < len - 1; i++)
+		{
+			tmp = lines[i];
+			lines[i] = g_strconcat(prefix, tmp, NULL);
+			g_free(tmp);
+		}
 	}
 	tmp = g_strjoinv(template_eol_char, lines);
 
@@ -538,7 +438,7 @@ gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 {
 	GString *template;
 
-	g_return_val_if_fail(doc != NULL, NULL);
+	g_return_val_if_fail(DOC_VALID(doc), NULL);
 	g_return_val_if_fail(licence_type == GEANY_TEMPLATE_GPL || licence_type == GEANY_TEMPLATE_BSD, NULL);
 
 	template = g_string_new(templates[licence_type]);
@@ -546,7 +446,7 @@ gchar *templates_get_template_licence(GeanyDocument *doc, gint licence_type)
 	templates_replace_default_dates(template);
 	templates_replace_command(template, DOC_FILENAME(doc), doc->file_type->name, NULL);
 
-	make_comment_block(template, doc->file_type->id, 8);
+	make_comment_block(template, doc->file_type->id, GEANY_TEMPLATES_INDENT);
 	convert_eol_characters(template, doc);
 
 	return g_string_free(template, FALSE);
@@ -565,7 +465,7 @@ static gchar *get_template_fileheader(GeanyFiletype *ft)
 		NULL);
 
 	/* we don't replace other wildcards here otherwise they would get done twice for files */
-	make_comment_block(template, ft->id, 8);
+	make_comment_block(template, ft->id, GEANY_TEMPLATES_INDENT);
 	return g_string_free(template, FALSE);
 }
 
@@ -584,33 +484,6 @@ gchar *templates_get_template_fileheader(gint filetype_idx, const gchar *fname)
 }
 
 
-/* old filetype templates - use file templates instead */
-gchar *templates_get_template_new_file(GeanyFiletype *ft)
-{
-	GString *ft_template;
-	gchar *file_header = NULL;
-
-	g_return_val_if_fail(ft != NULL, NULL);
-	g_return_val_if_fail(ft->id < GEANY_MAX_BUILT_IN_FILETYPES, NULL);
-
-	ft_template = g_string_new(ft_templates[ft->id]);
-	if (FILETYPE_ID(ft) == GEANY_FILETYPES_NONE)
-	{
-		replace_static_values(ft_template);
-	}
-	else
-	{	/* file template only used for new files */
-		file_header = get_template_fileheader(ft);
-		templates_replace_valist(ft_template, "{fileheader}", file_header, NULL);
-	}
-	templates_replace_common(ft_template, NULL, ft, NULL);
-	convert_eol_characters(ft_template, NULL);
-
-	g_free(file_header);
-	return g_string_free(ft_template, FALSE);
-}
-
-
 gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_name)
 {
 	GString *text;
@@ -622,7 +495,7 @@ gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_nam
 	templates_replace_default_dates(text);
 	templates_replace_command(text, DOC_FILENAME(doc), doc->file_type->name, func_name);
 
-	make_comment_block(text, doc->file_type->id, 3);
+	make_comment_block(text, doc->file_type->id, GEANY_TEMPLATES_INDENT);
 	convert_eol_characters(text, doc);
 
 	return g_string_free(text, FALSE);
@@ -631,9 +504,13 @@ gchar *templates_get_template_function(GeanyDocument *doc, const gchar *func_nam
 
 gchar *templates_get_template_changelog(GeanyDocument *doc)
 {
-	GString *result = g_string_new(templates[GEANY_TEMPLATE_CHANGELOG]);
-	const gchar *file_type_name = (doc != NULL) ? doc->file_type->name : "";
+	GString *result;
+	const gchar *file_type_name;
 
+	g_return_val_if_fail(DOC_VALID(doc), NULL);
+
+	result = g_string_new(templates[GEANY_TEMPLATE_CHANGELOG]);
+	file_type_name = (doc->file_type != NULL) ? doc->file_type->name : "";
 	replace_static_values(result);
 	templates_replace_default_dates(result);
 	templates_replace_command(result, DOC_FILENAME(doc), file_type_name, NULL);
@@ -657,10 +534,6 @@ void templates_free_templates(void)
 	{
 		g_free(templates[i]);
 	}
-	for (i = 0; i < GEANY_MAX_BUILT_IN_FILETYPES; i++)
-	{
-		g_free(ft_templates[i]);
-	}
 	/* destroy "New with template" sub menu items (in case we want to reload the templates) */
 	children = gtk_container_get_children(GTK_CONTAINER(new_with_template_menu));
 	foreach_list(item, children)
@@ -763,7 +636,7 @@ static gchar *run_command(const gchar *command, const gchar *file_name,
 		{
 			g_warning("templates_replace_command: %s", error->message);
 			g_error_free(error);
-			return NULL;
+			result = NULL;
 		}
 		g_strfreev(argv);
 		g_strfreev(env);
@@ -788,7 +661,7 @@ static void templates_replace_command(GString *text, const gchar *file_name,
 		while (*match != '}' && *match != '\0')
 			match++;
 
-		wildcard = g_strndup(cmd, match - cmd + 1);
+		wildcard = g_strndup(cmd, (gsize) (match - cmd + 1));
 		cmd = g_strndup(wildcard + 9, strlen(wildcard) - 10);
 
 		result = run_command(cmd, file_name, file_type, func_name);
