@@ -1,8 +1,8 @@
 /*
  *      utils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2011 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2011 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * $Id: utils.h 5906 2011-09-02 21:44:44Z colombanw $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -29,19 +27,28 @@
 #ifndef GEANY_UTILS_H
 #define GEANY_UTILS_H 1
 
+G_BEGIN_DECLS
 
 #include <time.h>
 
 
-/** Returns TRUE if @a ptr points to a non-zero value. */
-#define NZV(ptr) \
-	((ptr) && (ptr)[0])
+/** Returns @c TRUE if @a ptr is @c NULL or @c *ptr is @c FALSE. */
+#define EMPTY(ptr) \
+	(!(ptr) || !*(ptr))
 
-/**
- *  Frees @a ptr (if not @c NULL), then assigns @a result to it.
- *  @a result can be an expression using the 'old' value of @a ptr.
- *  It prevents a memory leak compared with: @code ptr = func(ptr); @endcode
+/** @deprecated 2013/08 - use @c !EMPTY() instead. */
+#ifndef GEANY_DISABLE_DEPRECATED
+#define NZV(ptr) (!EMPTY(ptr))
+#endif
+
+/** Assigns @a result to @a ptr, then frees the old value.
+ * @a result can be an expression using the 'old' value of @a ptr.
+ * E.g. @code SETPTR(str, g_strndup(str, 5)); @endcode
  **/
+#define SETPTR(ptr, result) \
+	do setptr(ptr, result) while (0)
+
+/** @deprecated 2011/11/15 - use SETPTR() instead. */
 #define setptr(ptr, result) \
 	{\
 		gpointer setptr_tmp = ptr;\
@@ -64,6 +71,13 @@
 		utils_get_setting_##type(home, group, key, default_val) :\
 		utils_get_setting_##type(sys, group, key, default_val)
 
+/* ignore the case of filenames and paths under WIN32, causes errors if not */
+#ifdef G_OS_WIN32
+#define utils_filenamecmp(a, b)	utils_str_casecmp((a), (b))
+#else
+#define utils_filenamecmp(a, b)	strcmp((a), (b))
+#endif
+
 
 /** Iterates all the items in @a array using pointers.
  * @param item pointer to an item in @a array.
@@ -168,8 +182,6 @@ gchar *utils_remove_ext_from_filename(const gchar *filename);
 
 gchar utils_brace_opposite(gchar ch);
 
-gchar *utils_get_hostname(void);
-
 gint utils_string_find(GString *haystack, gint start, gint end, const gchar *needle);
 
 gint utils_string_replace(GString *str, gint pos, gint len, const gchar *replace);
@@ -178,6 +190,9 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
 
 guint utils_string_replace_first(GString *haystack, const gchar *needle, const gchar *replace);
 
+guint utils_string_regex_replace_all(GString *haystack, GRegex *regex,
+		guint match_num, const gchar *replace, gboolean literal);
+
 void utils_str_replace_all(gchar **haystack, const gchar *needle, const gchar *replacement);
 
 gint utils_strpos(const gchar* haystack, const gchar *needle);
@@ -194,8 +209,6 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 
 gchar *utils_get_hex_from_color(GdkColor *color);
 
-guint utils_invert_color(guint color);
-
 const gchar *utils_get_default_dir_utf8(void);
 
 gchar *utils_get_current_file_dir_utf8(void);
@@ -205,7 +218,11 @@ void utils_beep(void);
 gchar *utils_make_human_readable_str(guint64 size, gulong block_size,
 									 gulong display_unit);
 
-gint utils_strtod(const gchar *source, gchar **end, gboolean with_route);
+gboolean utils_parse_color(const gchar *spec, GdkColor *color);
+
+gint utils_color_to_bgr(const GdkColor *color);
+
+gint utils_parse_color_to_bgr(const gchar *spec);
 
 gchar *utils_get_current_time_string(void);
 
@@ -253,10 +270,6 @@ gboolean utils_spawn_async(const gchar *dir, gchar **argv, gchar **env, GSpawnFl
 
 gint utils_str_casecmp(const gchar *s1, const gchar *s2);
 
-gchar *utils_build_path(const gchar *first, ...) G_GNUC_NULL_TERMINATED;
-
-gchar *utils_make_filename(const gchar *path, ...) G_GNUC_NULL_TERMINATED;
-
 gchar *utils_get_path_from_uri(const gchar *uri);
 
 gboolean utils_is_uri(const gchar *uri);
@@ -269,4 +282,10 @@ gchar *utils_str_remove_chars(gchar *string, const gchar *chars);
 
 gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...) G_GNUC_NULL_TERMINATED;
 
+GDate *utils_parse_date(const gchar *input);
+
+gchar *utils_parse_and_format_build_date(const gchar *input);
+
+G_END_DECLS
+
 #endif
