@@ -1,8 +1,8 @@
 /*
  *      win32.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2012 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2012 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
  *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *      GNU General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- *  $Id: win32.c 4914 2010-05-15 11:57:40Z eht16 $
+ *      You should have received a copy of the GNU General Public License along
+ *      with this program; if not, write to the Free Software Foundation, Inc.,
+ *      51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -33,6 +31,7 @@
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <commdlg.h>
+#include <shellapi.h>
 #include <shlobj.h>
 #include <io.h>
 #include <fcntl.h>
@@ -42,6 +41,7 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <glib/gstdio.h>
 #include <gdk/gdkwin32.h>
 
 #include "win32.h"
@@ -77,7 +77,8 @@ typedef struct _geany_win32_spawn geany_win32_spawn;
 
 static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **error);
 static HANDLE GetTempFileHandle(GError **error);
-static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error);
+static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline,
+		const TCHAR *dir, GError **error);
 static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **error);
 
 
@@ -121,12 +122,9 @@ static wchar_t *get_file_filters(void)
 
 	/* replace all "\t"s by \0 */
 	len = strlen(string);
-	for (i = 0; i < len; i++)
-	{
-		if (string[i] == '\t')
-			string[i] = '\0';
-	}
-	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
+	g_strdelimit(string, "\t", '\0');
+	g_assert(string[len - 1] == 0x0);
+	MultiByteToWideChar(CP_UTF8, 0, string, len, title, G_N_ELEMENTS(title));
 	g_free(string);
 
 	return title;
@@ -140,10 +138,12 @@ static wchar_t *get_file_filter_all_files(void)
 	gchar *filter;
 
 	/* create meta file filter "All files" */
-	filter = g_strdup_printf("%s\0*\0", _("All files"));
+	filter = g_strdup_printf("%s\t*\t", _("All files"));
 
-	len = strlen(_("All files")) + 3;
-	MultiByteToWideChar(CP_UTF8, 0, filter, len, title, sizeof(title));
+	len = strlen(filter);
+	g_strdelimit(filter, "\t", '\0');
+	g_assert(filter[len - 1] == 0x0);
+	MultiByteToWideChar(CP_UTF8, 0, filter, len, title, G_N_ELEMENTS(title));
 	g_free(filter);
 
 	return title;
@@ -153,29 +153,25 @@ static wchar_t *get_file_filter_all_files(void)
 static wchar_t *get_filters(gboolean project_files)
 {
 	gchar *string;
-	gint i, len;
+	gint len;
 	static wchar_t title[1024];
 
 	if (project_files)
 	{
 		string = g_strconcat(_("Geany project files"), "\t", "*." GEANY_PROJECT_EXT, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
+			_("All files"), "\t", "*", "\t", NULL);
 	}
 	else
 	{
 		string = g_strconcat(_("Executables"), "\t", "*.exe;*.bat;*.cmd", "\t",
-			filetypes[GEANY_FILETYPES_NONE]->title, "\t",
-			filetypes[GEANY_FILETYPES_NONE]->pattern[0], "\t", NULL);
+			_("All files"), "\t", "*", "\t", NULL);
 	}
 
 	/* replace all "\t"s by \0 */
 	len = strlen(string);
-	for (i = 0; i < len; i++)
-	{
-		if (string[i] == '\t') string[i] = '\0';
-	}
-	MultiByteToWideChar(CP_UTF8, 0, string, len, title, sizeof(title));
+	g_strdelimit(string, "\t", '\0');
+	g_assert(string[len - 1] == 0x0);
+	MultiByteToWideChar(CP_UTF8, 0, string, len, title, G_N_ELEMENTS(title));
 	g_free(string);
 
 	return title;
@@ -194,7 +190,7 @@ static wchar_t *get_dir_for_path(const gchar *utf8_filename)
 	else
 		result = g_path_get_dirname(utf8_filename);
 
-	MultiByteToWideChar(CP_UTF8, 0, result, -1, w_dir, sizeof(w_dir));
+	MultiByteToWideChar(CP_UTF8, 0, result, -1, w_dir, G_N_ELEMENTS(w_dir));
 
 	if (result != utf8_filename)
 		g_free(result);
@@ -242,13 +238,13 @@ gchar *win32_show_folder_dialog(GtkWidget *parent, const gchar *title, const gch
 	wchar_t fname[MAX_PATH];
 	wchar_t w_title[512];
 
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	if (parent == NULL)
 		parent = main_widgets.window;
 
 	memset(&bi, 0, sizeof bi);
-	bi.hwndOwner = GDK_WINDOW_HWND(parent->window);
+	bi.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	bi.pidlRoot = NULL;
 	bi.lpszTitle = w_title;
 	bi.lpfn = BrowseCallbackProc;
@@ -289,7 +285,7 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 
 	fname[0] = '\0';
 
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	if (parent == NULL)
 		parent = main_widgets.window;
@@ -301,7 +297,7 @@ gchar *win32_show_project_open_dialog(GtkWidget *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(parent->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	of.lpstrFilter = get_filters(project_file_filter);
 
 	of.lpstrCustomFilter = NULL;
@@ -352,9 +348,9 @@ gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title,
 	fname[0] = '\0';
 
 	if (initial_dir != NULL)
-		MultiByteToWideChar(CP_UTF8, 0, initial_dir, -1, w_dir, sizeof(w_dir));
+		MultiByteToWideChar(CP_UTF8, 0, initial_dir, -1, w_dir, G_N_ELEMENTS(w_dir));
 
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
@@ -363,7 +359,7 @@ gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 	of.lpstrFilter = get_file_filters();
 
 	of.lpstrCustomFilter = NULL;
@@ -424,21 +420,29 @@ gboolean win32_show_document_open_dialog(GtkWindow *parent, const gchar *title,
 
 
 gchar *win32_show_document_save_as_dialog(GtkWindow *parent, const gchar *title,
-										  const gchar *initial_file)
+										  GeanyDocument *doc)
 {
 	OPENFILENAMEW of;
 	gint retval;
 	gchar tmp[MAX_PATH];
 	wchar_t w_file[MAX_PATH];
 	wchar_t w_title[512];
-	guint x;
+	int n;
 
 	w_file[0] = '\0';
 
-	if (initial_file != NULL)
-		MultiByteToWideChar(CP_UTF8, 0, initial_file, -1, w_file, sizeof(w_file));
+	/* Convert the name of the file for of.lpstrFile */
+	n = MultiByteToWideChar(CP_UTF8, 0, DOC_FILENAME(doc), -1, w_file, G_N_ELEMENTS(w_file));
 
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+	/* If creating a new file name, convert and append the extension if any */
+	if (! doc->file_name && doc->file_type && doc->file_type->extension && n + 1 < G_N_ELEMENTS(w_file))
+	{
+		w_file[n - 1] = L'.';
+		MultiByteToWideChar(CP_UTF8, 0, doc->file_type->extension, -1, &w_file[n],
+				G_N_ELEMENTS(w_file) - n - 1);
+	}
+
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
@@ -447,7 +451,7 @@ gchar *win32_show_document_save_as_dialog(GtkWindow *parent, const gchar *title,
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 
 	of.lpstrFilter = get_file_filter_all_files();
 	of.lpstrCustomFilter = NULL;
@@ -488,14 +492,13 @@ gchar *win32_show_file_dialog(GtkWindow *parent, const gchar *title, const gchar
 	gchar tmp[MAX_PATH];
 	wchar_t w_file[MAX_PATH];
 	wchar_t w_title[512];
-	guint x;
 
 	w_file[0] = '\0';
 
 	if (initial_file != NULL)
-		MultiByteToWideChar(CP_UTF8, 0, initial_file, -1, w_file, sizeof(w_file));
+		MultiByteToWideChar(CP_UTF8, 0, initial_file, -1, w_file, G_N_ELEMENTS(w_file));
 
-	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, sizeof(w_title));
+	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	/* initialise file dialog info struct */
 	memset(&of, 0, sizeof of);
@@ -504,7 +507,7 @@ gchar *win32_show_file_dialog(GtkWindow *parent, const gchar *title, const gchar
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(GTK_WIDGET(parent)->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(GTK_WIDGET(parent)));
 
 	of.lpstrFile = w_file;
 	of.nMaxFile = 2048;
@@ -538,11 +541,15 @@ void win32_show_font_dialog(void)
 	CHOOSEFONT cf;
 	LOGFONT lf;        /* logical font structure */
 
+	memset(&lf, 0, sizeof lf);
+	/* TODO: init lf members */
+
 	memset(&cf, 0, sizeof cf);
 	cf.lStructSize = sizeof cf;
-	cf.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
+	cf.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 	cf.lpLogFont = &lf;
-	cf.Flags = CF_APPLY | CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
+	/* support CF_APPLY? */
+	cf.Flags = CF_NOSCRIPTSEL | CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_SCREENFONTS;
 
 	retval = ChooseFont(&cf);
 
@@ -566,9 +573,9 @@ void win32_show_color_dialog(const gchar *colour)
 	/* Initialize CHOOSECOLOR */
 	memset(&cc, 0, sizeof cc);
 	cc.lStructSize = sizeof(cc);
-	cc.hwndOwner = GDK_WINDOW_HWND(main_widgets.window->window);
+	cc.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 	cc.lpCustColors = (LPDWORD) acr_cust_clr;
-	cc.rgbResult = (colour != NULL) ? utils_strtod(colour, NULL, colour[0] == '#') : 0;
+	cc.rgbResult = (colour != NULL) ? utils_parse_color_to_bgr(colour) : 0;
 	cc.Flags = CC_FULLOPEN | CC_RGBINIT;
 
 	if (ChooseColor(&cc))
@@ -602,7 +609,7 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 		filename = g_find_program_in_path(field[0]);
 		if (filename != NULL && g_file_test(filename, G_FILE_TEST_EXISTS))
 		{
-			MultiByteToWideChar(CP_UTF8, 0, filename, -1, fname, sizeof(fname));
+			MultiByteToWideChar(CP_UTF8, 0, filename, -1, fname, G_N_ELEMENTS(fname));
 			g_free(filename);
 		}
 	}
@@ -614,7 +621,7 @@ void win32_show_pref_file_dialog(GtkEntry *item)
 #else
 	of.lStructSize = sizeof of;
 #endif
-	of.hwndOwner = GDK_WINDOW_HWND(ui_widgets.prefs_dialog->window);
+	of.hwndOwner = GDK_WINDOW_HWND(gtk_widget_get_window(ui_widgets.prefs_dialog));
 
 	of.lpstrFilter = get_filters(FALSE);
 	of.lpstrCustomFilter = NULL;
@@ -707,9 +714,9 @@ gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gcha
 	MultiByteToWideChar(CP_UTF8, 0, title, -1, w_title, G_N_ELEMENTS(w_title));
 
 	if (parent != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(parent->window);
+		parent_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(parent));
 	else if (main_widgets.window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
+		parent_hwnd = GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window));
 
 	/* display the message box */
 	rc = MessageBoxW(parent_hwnd, w_msg, w_title, t);
@@ -725,7 +732,7 @@ gboolean win32_message_dialog(GtkWidget *parent, GtkMessageType type, const gcha
 gint win32_check_write_permission(const gchar *dir)
 {
 	static wchar_t w_dir[MAX_PATH];
-	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, sizeof w_dir);
+	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, G_N_ELEMENTS(w_dir));
 	if (_waccess(w_dir, R_OK | W_OK) != 0)
 		return errno;
 	else
@@ -733,33 +740,6 @@ gint win32_check_write_permission(const gchar *dir)
 }
 
 
-/* Special dialog to ask for an action when closing an unsaved file */
-gint win32_message_dialog_unsaved(const gchar *msg)
-{
-	static wchar_t w_msg[512];
-	static wchar_t w_title[512];
-	HWND parent_hwnd = NULL;
-	gint ret;
-
-	/* convert the Unicode chars to wide chars */
-	MultiByteToWideChar(CP_UTF8, 0, msg, -1, w_msg, G_N_ELEMENTS(w_msg));
-	MultiByteToWideChar(CP_UTF8, 0, _("Question"), -1, w_title, G_N_ELEMENTS(w_title));
-
-	if (main_widgets.window != NULL)
-		parent_hwnd = GDK_WINDOW_HWND(main_widgets.window->window);
-
-	ret = MessageBoxW(parent_hwnd, w_msg, w_title, MB_YESNOCANCEL | MB_ICONQUESTION);
-	switch (ret)
-	{
-		case IDYES: ret = GTK_RESPONSE_YES; break;
-		case IDNO: ret = GTK_RESPONSE_NO; break;
-		case IDCANCEL: ret = GTK_RESPONSE_CANCEL; break;
-	}
-
-	return ret;
-}
-
-
 /* Just a simple wrapper function to open a browser window */
 void win32_open_browser(const gchar *uri)
 {
@@ -787,11 +767,52 @@ gboolean win32_get_exit_status(GPid child_pid)
 }
 
 
-static void debug_setup_console()
+static FILE *open_std_handle(DWORD handle, const char *mode)
+{
+	HANDLE lStdHandle;
+	int hConHandle;
+	FILE *fp;
+
+	lStdHandle = GetStdHandle(handle);
+	if (lStdHandle == INVALID_HANDLE_VALUE)
+	{
+		gchar *err = g_win32_error_message(GetLastError());
+		g_warning("GetStdHandle(%ld) failed: %s", (long)handle, err);
+		g_free(err);
+		return NULL;
+	}
+	hConHandle = _open_osfhandle((long)lStdHandle, _O_TEXT);
+	if (hConHandle == -1)
+	{
+		gchar *err = g_win32_error_message(GetLastError());
+		g_warning("_open_osfhandle(%ld, _O_TEXT) failed: %s", (long)lStdHandle, err);
+		g_free(err);
+		return NULL;
+	}
+	fp = _fdopen(hConHandle, mode);
+	if (! fp)
+	{
+		gchar *err = g_win32_error_message(GetLastError());
+		g_warning("_fdopen(%d, \"%s\") failed: %s", hConHandle, mode, err);
+		g_free(err);
+		return NULL;
+	}
+	if (setvbuf(fp, NULL, _IONBF, 0) != 0)
+	{
+		gchar *err = g_win32_error_message(GetLastError());
+		g_warning("setvbuf(%p, NULL, _IONBF, 0) failed: %s", fp, err);
+		g_free(err);
+		fclose(fp);
+		return NULL;
+	}
+
+	return fp;
+}
+
+
+static void debug_setup_console(void)
 {
 	static const WORD MAX_CONSOLE_LINES = 500;
-	gint	 hConHandle;
-	glong	 lStdHandle;
 	CONSOLE_SCREEN_BUFFER_INFO coninfo;
 	FILE	*fp;
 
@@ -804,18 +825,19 @@ static void debug_setup_console()
 	SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), coninfo.dwSize);
 
 	/* redirect unbuffered STDOUT to the console */
-	lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
-	hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
-	fp = _fdopen(hConHandle, "w");
-	*stdout = *fp;
-	setvbuf(stdout, NULL, _IONBF, 0);
+	fp = open_std_handle(STD_OUTPUT_HANDLE, "w");
+	if (fp)
+		*stdout = *fp;
 
 	/* redirect unbuffered STDERR to the console */
-	lStdHandle = (long)GetStdHandle(STD_ERROR_HANDLE);
-	hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);
-	fp = _fdopen(hConHandle, "w");
-	*stderr = *fp;
-	setvbuf(stderr, NULL, _IONBF, 0);
+	fp = open_std_handle(STD_ERROR_HANDLE, "w");
+	if (fp)
+		*stderr = *fp;
+
+	/* redirect unbuffered STDIN to the console */
+	fp = open_std_handle(STD_INPUT_HANDLE, "r");
+	if (fp)
+		*stdin = *fp;
 }
 
 
@@ -823,8 +845,8 @@ void win32_init_debug_code(void)
 {
 	if (app->debug_mode)
 	{
-		/* create a console window to get log messages on Windows */
-		/** TODO remove me? */
+		/* create a console window to get log messages on Windows,
+		 * especially useful when generating tags files */
 		debug_setup_console();
 		/* Enable GLib process spawn debug mode when Geany was started with the debug flag */
 		g_setenv("G_SPAWN_WIN32_DEBUG", "1", FALSE);
@@ -832,21 +854,28 @@ void win32_init_debug_code(void)
 }
 
 
-gchar *win32_get_hostname(void)
+static gchar *create_temp_file(void)
 {
-	gchar hostname[100];
-	DWORD size = sizeof(hostname);
+	gchar *name;
+	gint fd;
 
-	if (GetComputerName(hostname, &size))
-		return g_strdup(hostname);
+	fd = g_file_open_tmp("tmp_XXXXXX", &name, NULL);
+	if (fd == -1)
+		name = NULL;
 	else
-		return g_strdup("localhost");
+		close(fd);
+
+	return name;
 }
 
 
+/* Sometimes this blocks for 30s before aborting when there are several
+ * pages of (error) output and sometimes hangs - see the FIXME.
+ * Also gw_spawn.dwExitCode seems to be not set properly. */
 /* Process spawning implementation for Windows, by Pierre Joye.
  * Don't call this function directly, use utils_spawn_[a]sync() instead. */
-gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
+static
+gboolean _broken_win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
 					 gchar **std_out, gchar **std_err, gint *exit_status, GError **error)
 {
 	TCHAR  buffer[CMDSIZE]=TEXT("");
@@ -899,8 +928,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -913,8 +941,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("win32_spawn: Second CreateFile failed (%d)", (gint) GetLastError());
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 			g_free(msg);
 			return FALSE;
 		}
@@ -935,8 +962,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdout pipe creation failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -949,8 +975,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stderr pipe creation failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -963,8 +988,7 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("win32_spawn: Stdin pipe creation failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -982,8 +1006,6 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 	if (! fSuccess)
 	{
 		geany_debug("win32_spawn: Create process failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "Create process failed");
 		return FALSE;
 	}
 
@@ -1011,6 +1033,68 @@ gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags fl
 }
 
 
+/* Note: g_spawn is broken for receiving both stdio and stderr e.g. when
+ * running make and there are compile errors. See glib/giowin32.c header
+ * comment about Windows bugs, e.g. #338943 */
+/* Simple replacement for _broken_win32_spawn().
+ * flags is ignored, G_SPAWN_SEARCH_PATH is implied.
+ * Don't call this function directly, use utils_spawn_[a]sync() instead.
+ * Adapted from tm_workspace_create_global_tags(). */
+gboolean win32_spawn(const gchar *dir, gchar **argv, gchar **env, GSpawnFlags flags,
+					 gchar **std_out, gchar **std_err, gint *exit_status, GError **error)
+{
+	gint ret;
+	gboolean fail;
+	gchar *tmp_file = create_temp_file();
+	gchar *tmp_errfile = create_temp_file();
+	gchar *command;
+	gchar *locale_command;
+
+	if (env != NULL)
+	{
+		return _broken_win32_spawn(dir, argv, env, flags, std_out, std_err,
+			exit_status, error);
+	}
+	if (!tmp_file || !tmp_errfile)
+	{
+		g_warning("%s: Could not create temporary files!", G_STRFUNC);
+		return FALSE;
+	}
+	command = g_strjoinv(" ", argv);
+	SETPTR(command, g_strdup_printf("cmd.exe /S /C \"%s >%s 2>%s\"",
+		command, tmp_file, tmp_errfile));
+	locale_command = g_locale_from_utf8(command, -1, NULL, NULL, NULL);
+	if (! locale_command)
+		locale_command = g_strdup(command);
+	geany_debug("WIN32: actually running command:\n%s", command);
+	g_chdir(dir);
+	errno = 0;
+	ret = system(locale_command);
+	/* the command can return -1 as an exit code, so check errno also */
+	fail = ret == -1 && errno;
+	if (!fail)
+	{
+		if (std_out != NULL)
+			g_file_get_contents(tmp_file, std_out, NULL, NULL);
+		if (std_err != NULL)
+			g_file_get_contents(tmp_errfile, std_err, NULL, NULL);
+	}
+	else if (error)
+		g_set_error_literal(error, G_SPAWN_ERROR, errno, g_strerror(errno));
+
+	g_free(command);
+	g_free(locale_command);
+	g_unlink(tmp_file);
+	g_free(tmp_file);
+	g_unlink(tmp_errfile);
+	g_free(tmp_errfile);
+	if (exit_status)
+		*exit_status = ret;
+
+	return !fail;
+}
+
+
 static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **error)
 {
 	DWORD filesize;
@@ -1029,8 +1113,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Alloc failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_SPAWN_ERROR, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -1040,8 +1123,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: Cannot read tempfile");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
@@ -1050,8 +1132,7 @@ static gboolean GetContentFromHandle(HANDLE hFile, gchar **content, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetContentFromHandle: CloseHandle failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		g_free(buffer);
 		*content = NULL;
@@ -1074,7 +1155,8 @@ gchar *win32_expand_environment_variables(const gchar *str)
 }
 
 
-static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline, const TCHAR *dir, GError **error)
+static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline,
+		const TCHAR *dir, GError **error)
 {
 	PROCESS_INFORMATION piProcInfo;
 	STARTUPINFOW siStartInfo;
@@ -1098,8 +1180,8 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 	/* Expand environment variables like %blah%. */
 	expandedCmdline = win32_expand_environment_variables(szCmdline);
 
-	MultiByteToWideChar(CP_UTF8, 0, expandedCmdline, -1, w_commandline, sizeof(w_commandline));
-	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, sizeof(w_dir));
+	MultiByteToWideChar(CP_UTF8, 0, expandedCmdline, -1, w_commandline, G_N_ELEMENTS(w_commandline));
+	MultiByteToWideChar(CP_UTF8, 0, dir, -1, w_dir, G_N_ELEMENTS(w_dir));
 
 	/* Create the child process. */
 	bFuncRetn = CreateProcessW(NULL,
@@ -1119,28 +1201,30 @@ static gboolean CreateChildProcess(geany_win32_spawn *gw_spawn, TCHAR *szCmdline
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("CreateChildProcess: CreateProcess failed (%s)", msg);
-		if (*error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_SPAWN_ERROR_FAILED, "%s", msg);
 		g_free(msg);
 		return FALSE;
 	}
 	else
 	{
 		gint i;
-		DWORD dwStatus;
+		gsize ms = 30*1000;
 
-		for (i = 0; i < 2 && (dwStatus = WaitForSingleObject(piProcInfo.hProcess, 30*1000)) == WAIT_TIMEOUT; i++)
+		/* FIXME: this seems to timeout when there are many lines
+		 * to read - maybe because the child's pipe is full */
+		for (i = 0; i < 2 &&
+			WaitForSingleObject(piProcInfo.hProcess, ms) == WAIT_TIMEOUT; i++)
 		{
-			geany_debug("CreateChildProcess: CreateProcess failed");
+			ui_set_statusbar(FALSE, _("Process timed out after %.02f s!"), ms / 1000.0F);
+			geany_debug("CreateChildProcess: timed out");
 			TerminateProcess(piProcInfo.hProcess, WAIT_TIMEOUT); /* NOTE: This will not kill grandkids. */
 		}
 
-		if (GetExitCodeProcess(piProcInfo.hProcess, &gw_spawn->dwExitCode) != 0)
+		if (!GetExitCodeProcess(piProcInfo.hProcess, &gw_spawn->dwExitCode))
 		{
 			gchar *msg = g_win32_error_message(GetLastError());
 			geany_debug("GetExitCodeProcess failed: %s", msg);
-			if (error != NULL)
-				*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
+			g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_FAILED, "%s", msg);
 			g_free(msg);
 		}
 		CloseHandle(piProcInfo.hProcess);
@@ -1162,8 +1246,7 @@ static VOID ReadFromPipe(HANDLE hRead, HANDLE hWrite, HANDLE hFile, GError **err
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("ReadFromPipe: Closing handle failed");
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR_PIPE, "%s", msg);
 		g_free(msg);
 		return;
 	}
@@ -1198,8 +1281,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: GetTempPath failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1213,8 +1295,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileName failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1231,8 +1312,7 @@ static HANDLE GetTempFileHandle(GError **error)
 	{
 		gchar *msg = g_win32_error_message(GetLastError());
 		geany_debug("GetTempFileHandle: Second CreateFile failed (%d)", (gint) GetLastError());
-		if (error != NULL)
-			*error = g_error_new(G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
+		g_set_error(error, G_SPAWN_ERROR, G_FILE_ERROR, "%s", msg);
 		g_free(msg);
 		return NULL;
 	}
@@ -1314,7 +1394,7 @@ gchar *win32_get_shortcut_target(const gchar *file_name)
 	gchar *path = NULL;
 	wchar_t *wfilename = g_utf8_to_utf16(file_name, -1, NULL, NULL, NULL);
 
-	resolve_link(GDK_WINDOW_HWND(main_widgets.window->window), wfilename, &path);
+	resolve_link(GDK_WINDOW_HWND(gtk_widget_get_window(main_widgets.window)), wfilename, &path);
 	g_free(wfilename);
 
 	if (path == NULL)
@@ -1332,11 +1412,7 @@ void win32_set_working_directory(const gchar *dir)
 
 gchar *win32_get_installation_dir(void)
 {
-#if GLIB_CHECK_VERSION(2, 16, 0)
 	return g_win32_get_package_installation_directory_of_module(NULL);
-#else
-	return g_win32_get_package_installation_directory(NULL, NULL);
-#endif
 }
 
 
